@@ -41,6 +41,7 @@
 #include "clutter-event.h"
 #include "clutter-backend.h"
 #include "clutter-main.h"
+#include "clutter-master-clock.h"
 #include "clutter-feature.h"
 #include "clutter-actor.h"
 #include "clutter-stage.h"
@@ -92,7 +93,9 @@ static const GDebugKey clutter_debug_keys[] = {
   { "script", CLUTTER_DEBUG_SCRIPT },
   { "shader", CLUTTER_DEBUG_SHADER },
   { "multistage", CLUTTER_DEBUG_MULTISTAGE },
-  { "animation", CLUTTER_DEBUG_ANIMATION }
+  { "animation", CLUTTER_DEBUG_ANIMATION },
+  { "layout", CLUTTER_DEBUG_LAYOUT },
+  { "nop-picking", CLUTTER_DEBUG_NOP_PICKING }
 };
 #endif /* CLUTTER_ENABLE_DEBUG */
 
@@ -117,7 +120,7 @@ clutter_get_show_fps (void)
 void
 _clutter_stage_maybe_relayout (ClutterActor *stage)
 {
-  ClutterUnit natural_width, natural_height;
+  gfloat natural_width, natural_height;
   ClutterActorBox box = { 0, };
 
   /* avoid reentrancy */
@@ -138,10 +141,10 @@ _clutter_stage_maybe_relayout (ClutterActor *stage)
       box.y2 = natural_height;
 
       CLUTTER_NOTE (ACTOR, "Allocating (0, 0 - %d, %d) for the stage",
-                    CLUTTER_UNITS_TO_DEVICE (natural_width),
-                    CLUTTER_UNITS_TO_DEVICE (natural_height));
+                    (int) natural_width,
+                    (int) natural_height);
 
-      clutter_actor_allocate (stage, &box, FALSE);
+      clutter_actor_allocate (stage, &box, CLUTTER_ALLOCATION_NONE);
 
       CLUTTER_UNSET_PRIVATE_FLAGS (stage, CLUTTER_ACTOR_IN_RELAYOUT);
     }
@@ -150,44 +153,41 @@ _clutter_stage_maybe_relayout (ClutterActor *stage)
 void
 _clutter_stage_maybe_setup_viewport (ClutterStage *stage)
 {
-  if (CLUTTER_PRIVATE_FLAGS (stage) & CLUTTER_ACTOR_SYNC_MATRICES)
+  if ((CLUTTER_PRIVATE_FLAGS (stage) & CLUTTER_ACTOR_SYNC_MATRICES) &&
+      !(CLUTTER_PRIVATE_FLAGS (stage) & CLUTTER_STAGE_IN_RESIZE))
     {
       ClutterPerspective perspective;
-      guint width, height;
+      gfloat width, height;
 
-      clutter_actor_get_size (CLUTTER_ACTOR (stage), &width, &height);
+      clutter_actor_get_preferred_size (CLUTTER_ACTOR (stage),
+                                        NULL, NULL,
+                                        &width, &height);
       clutter_stage_get_perspective (stage, &perspective);
 
-      CLUTTER_NOTE (PAINT, "Setting up the viewport");
+      CLUTTER_NOTE (PAINT,
+                    "Setting up the viewport { w:%.2f, h:%.2f }",
+                    width, height);
 
-      cogl_setup_viewport (width, height,
-			   perspective.fovy,
-			   perspective.aspect,
-			   perspective.z_near,
-			   perspective.z_far);
+      _cogl_setup_viewport (width, height,
+                            perspective.fovy,
+                            perspective.aspect,
+                            perspective.z_near,
+                            perspective.z_far);
 
       CLUTTER_UNSET_PRIVATE_FLAGS (stage, CLUTTER_ACTOR_SYNC_MATRICES);
     }
 }
 
-/**
- * clutter_redraw:
- *
- * Forces a redraw of the entire stage. Applications should never use this
- * function, but queue a redraw using clutter_actor_queue_redraw().
- */
 void
-clutter_redraw (ClutterStage *stage)
+_clutter_do_redraw (ClutterStage *stage)
 {
   ClutterMainContext *ctx;
-  static GTimer      *timer = NULL;
-  static guint        timer_n_frames = 0;
+  ClutterMasterClock *master_clock;
+  static GTimer *timer = NULL;
+  static guint timer_n_frames = 0;
 
-  ctx  = clutter_context_get_default ();
-
-  CLUTTER_TIMESTAMP (SCHEDULER, "Redraw start for stage:%p", stage);
-  CLUTTER_NOTE (PAINT, " Redraw enter for stage:%p", stage);
-  CLUTTER_NOTE (MULTISTAGE, "Redraw called for stage:%p", stage);
+  ctx  = _clutter_context_get_default ();
+  master_clock = _clutter_master_clock_get_default ();
 
   /* Before we can paint, we have to be sure we have the latest layout */
   _clutter_stage_maybe_relayout (CLUTTER_ACTOR (stage));
@@ -208,7 +208,7 @@ clutter_redraw (ClutterStage *stage)
 
   /* Call through to the actual backend to do the painting down from
    * the stage. It will likely need to swap buffers, vblank sync etc
-   * which will be windowing system dependant.
+   * which will be windowing system dependent
   */
   _clutter_backend_redraw (ctx->backend, stage);
 
@@ -225,10 +225,26 @@ clutter_redraw (ClutterStage *stage)
 	}
     }
 
-  CLUTTER_NOTE (PAINT, " Redraw leave for stage:%p", stage);
   CLUTTER_TIMESTAMP (SCHEDULER, "Redraw finish for stage:%p", stage);
 }
 
+/**
+ * clutter_redraw:
+ *
+ * Forces a redraw of the entire stage. Applications should never use this
+ * function, but queue a redraw using clutter_actor_queue_redraw().
+ *
+ * This function should only be used by libraries integrating Clutter from
+ * within another toolkit.
+ */
+void
+clutter_redraw (ClutterStage *stage)
+{
+  g_return_if_fail (CLUTTER_IS_STAGE (stage));
+
+  clutter_stage_ensure_redraw (stage);
+}
+
 /**
  * clutter_set_motion_events_enabled:
  * @enable: %TRUE to enable per-actor motion events
@@ -249,7 +265,7 @@ clutter_redraw (ClutterStage *stage)
 void
 clutter_set_motion_events_enabled (gboolean enable)
 {
-  ClutterMainContext *context = clutter_context_get_default ();
+  ClutterMainContext *context = _clutter_context_get_default ();
 
   context->motion_events_per_actor = enable;
 }
@@ -266,7 +282,7 @@ clutter_set_motion_events_enabled (gboolean enable)
 gboolean
 clutter_get_motion_events_enabled (void)
 {
-  ClutterMainContext *context = clutter_context_get_default ();
+  ClutterMainContext *context = _clutter_context_get_default ();
 
   return context->motion_events_per_actor;
 }
@@ -281,7 +297,7 @@ static inline void init_bits (void)
   if (G_LIKELY (done))
     return;
 
-  ctx = clutter_context_get_default ();
+  ctx = _clutter_context_get_default ();
 
   done = TRUE;
 }
@@ -290,8 +306,9 @@ void
 _clutter_id_to_color (guint id, ClutterColor *col)
 {
   ClutterMainContext *ctx;
-  gint                red, green, blue;
-  ctx = clutter_context_get_default ();
+  gint red, green, blue;
+
+  ctx = _clutter_context_get_default ();
 
   /* compute the numbers we'll store in the components */
   red   = (id >> (ctx->fb_g_mask_used+ctx->fb_b_mask_used))
@@ -328,7 +345,7 @@ _clutter_pixel_to_id (guchar pixel[4])
   gint  red, green, blue;
   guint id;
 
-  ctx = clutter_context_get_default ();
+  ctx = _clutter_context_get_default ();
 
   /* reduce the pixel components to the number of bits actually used of the
    * 8bits.
@@ -362,20 +379,27 @@ _clutter_do_pick (ClutterStage   *stage,
   guint32             id;
   GLboolean           dither_was_on;
 
-  context = clutter_context_get_default ();
+  if (clutter_debug_flags & CLUTTER_DEBUG_NOP_PICKING)
+    return CLUTTER_ACTOR (stage);
+
+  context = _clutter_context_get_default ();
 
   _clutter_backend_ensure_context (context->backend, stage);
 
   /* needed for when a context switch happens */
   _clutter_stage_maybe_setup_viewport (stage);
 
+  cogl_clip_push_window_rect (x, y, 1, 1);
   cogl_color_set_from_4ub (&white, 255, 255, 255, 255);
   cogl_disable_fog ();
-  cogl_clear (&white);
+  cogl_clear (&white,
+	      COGL_BUFFER_BIT_COLOR |
+	      COGL_BUFFER_BIT_DEPTH);
 
   /* Disable dithering (if any) when doing the painting in pick mode */
   dither_was_on = glIsEnabled (GL_DITHER);
-  glDisable (GL_DITHER);
+  if (dither_was_on)
+    glDisable (GL_DITHER);
 
   /* Render the entire scence in pick mode - just single colored silhouette's
    * are drawn offscreen (as we never swap buffers)
@@ -383,14 +407,13 @@ _clutter_do_pick (ClutterStage   *stage,
   context->pick_mode = mode;
   clutter_actor_paint (CLUTTER_ACTOR (stage));
   context->pick_mode = CLUTTER_PICK_NONE;
+  cogl_clip_pop ();
 
   /* Calls should work under both GL and GLES, note GLES needs RGBA */
   glGetIntegerv(GL_VIEWPORT, viewport);
 
-  /* Below to be safe, particularly on GL ES. an EGL wait call or full
-   * could be nicer.
-  */
-  glFinish();
+  /* Make sure Cogl flushes any batched geometry to the GPU driver */
+  cogl_flush ();
 
   /* Read the color of the screen co-ords pixel */
   glReadPixels (x, viewport[3] - y -1, 1, 1, GL_RGBA, GL_UNSIGNED_BYTE, pixel);
@@ -1039,21 +1062,31 @@ clutter_get_debug_enabled (void)
 #endif
 }
 
-ClutterMainContext *
-clutter_context_get_default (void)
+gboolean
+_clutter_context_is_initialized (void)
 {
-  if (G_UNLIKELY(!ClutterCntx))
+  if (ClutterCntx == NULL)
+    return FALSE;
+
+  return ClutterCntx->is_initialized;
+}
+
+ClutterMainContext *
+_clutter_context_get_default (void)
+{
+  if (G_UNLIKELY (ClutterCntx == NULL))
     {
       ClutterMainContext *ctx;
 
       ClutterCntx = ctx = g_new0 (ClutterMainContext, 1);
+
       ctx->backend = g_object_new (_clutter_backend_impl_get_type (), NULL);
 
       ctx->is_initialized = FALSE;
       ctx->motion_events_per_actor = TRUE;
 
 #ifdef CLUTTER_ENABLE_DEBUG
-      ctx->timer          =  g_timer_new ();
+      ctx->timer = g_timer_new ();
       g_timer_start (ctx->timer);
 #endif
     }
@@ -1076,7 +1109,7 @@ clutter_get_timestamp (void)
   ClutterMainContext *ctx;
   gdouble seconds;
 
-  ctx = clutter_context_get_default ();
+  ctx = _clutter_context_get_default ();
 
   /* FIXME: may need a custom timer for embedded setups */
   seconds = g_timer_elapsed (ctx->timer, NULL);
@@ -1142,7 +1175,7 @@ clutter_init_real (GError **error)
   /* Note, creates backend if not already existing, though parse args will
    * have likely created it
    */
-  ctx = clutter_context_get_default ();
+  ctx = _clutter_context_get_default ();
   backend = ctx->backend;
 
   if (!ctx->options_parsed)
@@ -1285,7 +1318,7 @@ pre_parse_hook (GOptionContext  *context,
     g_warning ("Locale not supported by C library.\n"
                "Using the fallback 'C' locale.");
 
-  clutter_context = clutter_context_get_default ();
+  clutter_context = _clutter_context_get_default ();
 
   clutter_context->id_pool = clutter_id_pool_new (256);
 
@@ -1347,7 +1380,7 @@ post_parse_hook (GOptionContext  *context,
   if (clutter_is_initialized)
     return TRUE;
 
-  clutter_context = clutter_context_get_default ();
+  clutter_context = _clutter_context_get_default ();
   backend = clutter_context->backend;
   g_assert (CLUTTER_IS_BACKEND (backend));
 
@@ -1412,7 +1445,7 @@ clutter_get_option_group (void)
 
   clutter_base_init ();
 
-  context = clutter_context_get_default ();
+  context = _clutter_context_get_default ();
 
   group = g_option_group_new ("clutter",
                               _("Clutter Options"),
@@ -1455,7 +1488,7 @@ clutter_get_option_group_without_init (void)
 
   clutter_base_init ();
 
-  context = clutter_context_get_default ();
+  context = _clutter_context_get_default ();
   context->defer_display_setup = TRUE;
 
   group = clutter_get_option_group ();
@@ -1514,7 +1547,7 @@ clutter_init_with_args (int            *argc,
 
   clutter_base_init ();
 
-  ctx = clutter_context_get_default ();
+  ctx = _clutter_context_get_default ();
 
   if (!ctx->defer_display_setup)
     {
@@ -1619,7 +1652,7 @@ clutter_init (int    *argc,
 
   clutter_base_init ();
 
-  ctx = clutter_context_get_default ();
+  ctx = _clutter_context_get_default ();
 
   if (!ctx->defer_display_setup)
     {
@@ -1671,7 +1704,7 @@ event_click_count_generate (ClutterEvent *event)
   guint           double_click_time;
   guint           double_click_distance;
 
-  backend = clutter_context_get_default ()->backend;
+  backend = _clutter_context_get_default ()->backend;
   double_click_distance = clutter_backend_get_double_click_distance (backend);
   double_click_time = clutter_backend_get_double_click_time (backend);
 
@@ -1850,41 +1883,6 @@ unset_motion_last_actor (ClutterActor *actor, ClutterInputDevice *dev)
     dev->motion_last_actor = NULL;
 }
 
-static ClutterInputDevice * clutter_event_get_device (ClutterEvent *event);
-
-/* This function should perhaps be public and in clutter-event.c ?
- */
-static ClutterInputDevice *
-clutter_event_get_device (ClutterEvent *event)
-{
-  g_return_val_if_fail (event != NULL, NULL);
-
-  switch (event->type)
-    {
-    case CLUTTER_NOTHING:
-    case CLUTTER_STAGE_STATE:
-    case CLUTTER_DESTROY_NOTIFY:
-    case CLUTTER_CLIENT_MESSAGE:
-    case CLUTTER_DELETE:
-    case CLUTTER_ENTER:
-    case CLUTTER_LEAVE:
-      return NULL;
-      break;
-    case CLUTTER_BUTTON_PRESS:
-    case CLUTTER_BUTTON_RELEASE:
-      return event->button.device;
-    case CLUTTER_MOTION:
-      return event->motion.device;
-    case CLUTTER_SCROLL:
-      return event->scroll.device;
-      break;
-    case CLUTTER_KEY_PRESS:
-    case CLUTTER_KEY_RELEASE:
-      break;
-    }
-  return NULL;
-}
-
 static void
 set_motion_last_actor (ClutterActor       *motion_current_actor,
                        ClutterInputDevice *device)
@@ -1931,8 +1929,8 @@ generate_enter_leave_events (ClutterEvent *event)
     {
       if (motion_current_actor)
         {
-          gint         x, y;
           ClutterEvent cev;
+          gfloat x, y;
 
           cev.crossing.device  = device;
           clutter_event_get_coords (event, &x, &y);
@@ -1981,6 +1979,29 @@ generate_enter_leave_events (ClutterEvent *event)
  */
 void
 clutter_do_event (ClutterEvent *event)
+{
+  if (!event->any.stage)
+    return;
+
+  /* Instead of processing events when received, we queue them up to
+   * handle per-frame before animations, layout, and drawing.
+   *
+   * This gives us the chance to reliably compress motion events
+   * because we've "looked ahead" and know all motion events that
+   * will occur before drawing the frame.
+   */
+  _clutter_stage_queue_event (event->any.stage, event);
+}
+
+/**
+ * _clutter_process_event
+ * @event: a #ClutterEvent.
+ *
+ * Does the actual work of processing an event that was queued earlier
+ * out of clutter_do_event().
+ */
+void
+_clutter_process_event (ClutterEvent *event)
 {
   /* FIXME: This should probably be clutter_cook_event() - it would
    * take a raw event from the backend and 'cook' it so its more tasty.
@@ -1990,10 +2011,8 @@ clutter_do_event (ClutterEvent *event)
   ClutterBackend      *backend;
   ClutterActor        *stage;
   ClutterInputDevice  *device = NULL;
-  static gint32        motion_last_time = 0L;
-  gint32               local_motion_time;
 
-  context = clutter_context_get_default ();
+  context = _clutter_context_get_default ();
   backend = context->backend;
   stage   = CLUTTER_ACTOR(event->any.stage);
 
@@ -2068,52 +2087,6 @@ clutter_do_event (ClutterEvent *event)
       case CLUTTER_MOTION:
         device = event->motion.device;
 
-        if (device)
-          local_motion_time = device->motion_last_time;
-        else
-          local_motion_time = motion_last_time;
-
-        /* avoid rate throttling for synthetic motion events or if
-         * the per-actor events are disabled
-         */
-        if (!(event->any.flags & CLUTTER_EVENT_FLAG_SYNTHETIC) ||
-            !context->motion_events_per_actor)
-          {
-            gint32 frame_rate, delta;
-
-            /* avoid issuing too many motion events, which leads to many
-             * redraws in pick mode (performance penalty)
-             */
-            frame_rate = clutter_get_motion_events_frequency ();
-            delta = 1000 / frame_rate;
-
-            CLUTTER_NOTE (EVENT,
-                  "skip motion event: %s (last:%d, delta:%d, time:%d)",
-                  (event->any.time < (local_motion_time + delta) ? "yes" : "no"),
-                  local_motion_time,
-                  delta,
-                  event->any.time);
-
-            /* we need to guard against roll-overs and the
-             * case where the time is rolled backwards and
-             * the backend is not ensuring a monotonic clock
-             * for the events.
-             *
-             * see:
-             *   http://bugzilla.openedhand.com/show_bug.cgi?id=1130
-             */
-            if (event->any.time >= local_motion_time &&
-                event->any.time < (local_motion_time + delta))
-              break;
-            else
-              local_motion_time = event->any.time;
-          }
-
-        if (device)
-          device->motion_last_time = local_motion_time;
-        else
-          motion_last_time = local_motion_time;
-
         /* Only stage gets motion events if clutter_set_motion_events is TRUE,
          * and the event is not a synthetic event with source set.
          */
@@ -2153,7 +2126,7 @@ clutter_do_event (ClutterEvent *event)
       case CLUTTER_SCROLL:
         {
           ClutterActor *actor;
-          gint          x,y;
+          gfloat x, y;
 
           clutter_event_get_coords (event, &x, &y);
 
@@ -2170,7 +2143,7 @@ clutter_do_event (ClutterEvent *event)
                   if (event->type == CLUTTER_BUTTON_RELEASE)
                     {
                       CLUTTER_NOTE (EVENT,
-                                    "Release off stage received at %i, %i",
+                                    "Release off stage received at %.2f, %.2f",
                                     x, y);
 
                       event->button.source = stage;
@@ -2196,12 +2169,14 @@ clutter_do_event (ClutterEvent *event)
 
 
           /* FIXME: for an optimisation should check if there are
-           * actually any reactive actors and avoid the pick all togeather
+           * actually any reactive actors and avoid the pick all together
            * (signalling just the stage). Should be big help for gles.
            */
 
-          CLUTTER_NOTE (EVENT, "Reactive event received at %i, %i - actor: %p",
-                        x, y, actor);
+          CLUTTER_NOTE (EVENT,
+                        "Reactive event received at %.2f, %.2f - actor: %p",
+                        x, y,
+                        actor);
 
           /* Create, enter/leave events if needed */
           generate_enter_leave_events (event);
@@ -2261,7 +2236,7 @@ clutter_get_actor_by_gid (guint32 id)
 {
   ClutterMainContext *context;
 
-  context = clutter_context_get_default ();
+  context = _clutter_context_get_default ();
 
   g_return_val_if_fail (context != NULL, NULL);
 
@@ -2293,11 +2268,7 @@ clutter_base_init (void)
 /**
  * clutter_get_default_frame_rate:
  *
- * Retrieves the default frame rate used when creating #ClutterTimeline<!--
- * -->s.
- *
- * This value is also used to compute the default frequency of motion
- * events.
+ * Retrieves the default frame rate. See clutter_set_default_frame_rate().
  *
  * Return value: the default frame rate
  *
@@ -2308,7 +2279,7 @@ clutter_get_default_frame_rate (void)
 {
   ClutterMainContext *context;
 
-  context = clutter_context_get_default ();
+  context = _clutter_context_get_default ();
 
   return context->frame_rate;
 }
@@ -2317,8 +2288,10 @@ clutter_get_default_frame_rate (void)
  * clutter_set_default_frame_rate:
  * @frames_per_sec: the new default frame rate
  *
- * Sets the default frame rate to be used when creating #ClutterTimeline<!--
- * -->s
+ * Sets the default frame rate. This frame rate will be used to limit
+ * the number of frames drawn if Clutter is not able to synchronize
+ * with the vertical refresh rate of the display. When synchronization
+ * is possible, this value is ignored.
  *
  * Since: 0.6
  */
@@ -2327,7 +2300,7 @@ clutter_set_default_frame_rate (guint frames_per_sec)
 {
   ClutterMainContext *context;
 
-  context = clutter_context_get_default ();
+  context = _clutter_context_get_default ();
 
   if (context->frame_rate != frames_per_sec)
     context->frame_rate = frames_per_sec;
@@ -2341,7 +2314,7 @@ on_pointer_grab_weak_notify (gpointer data,
   ClutterInputDevice *dev = (ClutterInputDevice *)data;
   ClutterMainContext *context;
 
-  context = clutter_context_get_default ();
+  context = _clutter_context_get_default ();
 
   if (dev)
     {
@@ -2382,7 +2355,7 @@ clutter_grab_pointer (ClutterActor *actor)
 
   g_return_if_fail (actor == NULL || CLUTTER_IS_ACTOR (actor));
 
-  context = clutter_context_get_default ();
+  context = _clutter_context_get_default ();
 
   if (context->pointer_grab_actor == actor)
     return;
@@ -2499,7 +2472,7 @@ ClutterActor *
 clutter_get_pointer_grab (void)
 {
   ClutterMainContext *context;
-  context = clutter_context_get_default ();
+  context = _clutter_context_get_default ();
 
   return context->pointer_grab_actor;
 }
@@ -2511,7 +2484,7 @@ on_keyboard_grab_weak_notify (gpointer data,
 {
   ClutterMainContext *context;
 
-  context = clutter_context_get_default ();
+  context = _clutter_context_get_default ();
   context->keyboard_grab_actor = NULL;
 
   clutter_ungrab_keyboard ();
@@ -2542,7 +2515,7 @@ clutter_grab_keyboard (ClutterActor *actor)
 
   g_return_if_fail (actor == NULL || CLUTTER_IS_ACTOR (actor));
 
-  context = clutter_context_get_default ();
+  context = _clutter_context_get_default ();
 
   if (context->keyboard_grab_actor == actor)
     return;
@@ -2591,66 +2564,12 @@ ClutterActor *
 clutter_get_keyboard_grab (void)
 {
   ClutterMainContext *context;
-  context = clutter_context_get_default ();
+
+  context = _clutter_context_get_default ();
 
   return context->keyboard_grab_actor;
 }
 
-/**
- * clutter_get_motion_events_frequency:
- *
- * Retrieves the number of motion events per second that are delivered
- * to the stage.
- *
- * See clutter_set_motion_events_frequency().
- *
- * Return value: the number of motion events per second
- *
- * Since: 0.6
- */
-guint
-clutter_get_motion_events_frequency (void)
-{
-  ClutterMainContext *context = clutter_context_get_default ();
-
-  if (G_LIKELY (context->motion_frequency == 0))
-    {
-      guint frequency;
-
-      frequency = clutter_default_fps / 4;
-      frequency = CLAMP (frequency, 20, 45);
-
-      return frequency;
-    }
-  else
-    return context->motion_frequency;
-}
-
-/**
- * clutter_set_motion_events_frequency:
- * @frequency: the number of motion events per second, or 0 for the
- *   default value
- *
- * Sets the motion events frequency. Setting this to a non-zero value
- * will override the default setting, so it should be rarely used.
- *
- * Motion events are delivered from the default backend to the stage
- * and are used to generate the enter/leave events pair. This might lead
- * to a performance penalty due to the way the actors are identified.
- * Using this function is possible to reduce the frequency of the motion
- * events delivery to the stage.
- *
- * Since: 0.6
- */
-void
-clutter_set_motion_events_frequency (guint frequency)
-{
-  ClutterMainContext *context = clutter_context_get_default ();
-
-  /* never allow the motion events to exceed the default frame rate */
-  context->motion_frequency = CLAMP (frequency, 1, clutter_default_fps);
-}
-
 /**
  * clutter_clear_glyph_cache:
  *
@@ -2773,7 +2692,7 @@ clutter_get_input_device_for_id (gint id)
   ClutterInputDevice *device = NULL;
   ClutterMainContext  *context;
 
-  context = clutter_context_get_default ();
+  context = _clutter_context_get_default ();
 
   for (item = context->input_devices;
        item != NULL;
@@ -2808,3 +2727,162 @@ clutter_get_font_map (void)
 
   return NULL;
 }
+
+typedef struct _ClutterRepaintFunction
+{
+  guint id;
+  GSourceFunc func;
+  gpointer data;
+  GDestroyNotify notify;
+} ClutterRepaintFunction;
+
+/**
+ * clutter_threads_remove_repaint_func:
+ * @handle_id: an unsigned integer greater than zero
+ *
+ * Removes the repaint function with @handle_id as its id
+ *
+ * Since: 1.0
+ */
+void
+clutter_threads_remove_repaint_func (guint handle_id)
+{
+  ClutterRepaintFunction *repaint_func;
+  ClutterMainContext *context;
+  GList *l;
+
+  g_return_if_fail (handle_id > 0);
+
+  context = CLUTTER_CONTEXT ();
+  l = context->repaint_funcs;
+  while (l != NULL)
+    {
+      repaint_func = l->data;
+
+      if (repaint_func->id == handle_id)
+        {
+          context->repaint_funcs =
+            g_list_remove_link (context->repaint_funcs, l);
+
+          g_list_free (l);
+
+          if (repaint_func->notify)
+            repaint_func->notify (repaint_func->data);
+
+          g_slice_free (ClutterRepaintFunction, repaint_func);
+
+          return;
+        }
+
+      l = l->next;
+    }
+}
+
+/**
+ * clutter_threads_add_repaint_func:
+ * @func: the function to be called within the paint cycle
+ * @data: data to be passed to the function, or %NULL
+ * @notify: function to be called when removing the repaint
+ *    function, or %NULL
+ *
+ * Adds a function to be called whenever Clutter is repainting a Stage.
+ * If the function returns %FALSE it is automatically removed from the
+ * list of repaint functions and will not be called again.
+ *
+ * This function is guaranteed to be called from within the same thread
+ * that called clutter_main(), and while the Clutter lock is being held.
+ *
+ * A repaint function is useful to ensure that an update of the scenegraph
+ * is performed before the scenegraph is repainted; for instance, uploading
+ * a frame from a video into a #ClutterTexture.
+ *
+ * When the repaint function is removed (either because it returned %FALSE
+ * or because clutter_threads_remove_repaint_func() has been called) the
+ * @notify function will be called, if any is set.
+ *
+ * Return value: the ID (greater than 0) of the repaint function. You
+ *   can use the returned integer to remove the repaint function by
+ *   calling clutter_threads_remove_repaint_func().
+ *
+ * Since: 1.0
+ */
+guint
+clutter_threads_add_repaint_func (GSourceFunc    func,
+                                  gpointer       data,
+                                  GDestroyNotify notify)
+{
+  static guint repaint_id = 1;
+  ClutterMainContext *context;
+  ClutterRepaintFunction *repaint_func;
+
+  g_return_val_if_fail (func != NULL, 0);
+
+  context = CLUTTER_CONTEXT ();
+
+  /* XXX lock the context */
+
+  repaint_func = g_slice_new (ClutterRepaintFunction);
+
+  repaint_func->id = repaint_id++;
+  repaint_func->func = func;
+  repaint_func->data = data;
+  repaint_func->notify = notify;
+
+  context->repaint_funcs = g_list_prepend (context->repaint_funcs,
+                                           repaint_func);
+
+  /* XXX unlock the context */
+
+  return repaint_func->id;
+}
+
+/*
+ * _clutter_run_repaint_functions:
+ *
+ * Executes the repaint functions added using the
+ * clutter_threads_add_repaint_func() function.
+ *
+ * Must be called before calling clutter_redraw() and
+ * with the Clutter thread lock held.
+ */
+void
+_clutter_run_repaint_functions (void)
+{
+  ClutterMainContext *context = CLUTTER_CONTEXT ();
+  ClutterRepaintFunction *repaint_func;
+  GList *reinvoke_list, *l;
+
+  if (context->repaint_funcs == NULL)
+    return;
+
+  reinvoke_list = NULL;
+
+  /* consume the whole list while we execute the functions */
+  while (context->repaint_funcs)
+    {
+      gboolean res = FALSE;
+
+      repaint_func = context->repaint_funcs->data;
+
+      l = context->repaint_funcs;
+      context->repaint_funcs =
+        g_list_remove_link (context->repaint_funcs, context->repaint_funcs);
+
+      g_list_free (l);
+
+      res = repaint_func->func (repaint_func->data);
+
+      if (res)
+        reinvoke_list = g_list_prepend (reinvoke_list, repaint_func);
+      else
+        {
+          if (repaint_func->notify)
+            repaint_func->notify (repaint_func->data);
+
+          g_slice_free (ClutterRepaintFunction, repaint_func);
+        }
+    }
+
+  if (reinvoke_list)
+    context->repaint_funcs = reinvoke_list;
+}
