@@ -44,9 +44,11 @@
 #include "clutter-event.h"
 #include "clutter-feature.h"
 #include "clutter-id-pool.h"
+#include "clutter-master-clock.h"
 #include "clutter-stage-manager.h"
 #include "clutter-stage-window.h"
 #include "clutter-stage.h"
+#include "clutter-timeline.h"
 
 G_BEGIN_DECLS
 
@@ -62,19 +64,16 @@ typedef enum {
                                           */
   CLUTTER_ACTOR_IN_PAINT         = 1 << 4, /* Used to avoid recursion */
   CLUTTER_ACTOR_IN_RELAYOUT      = 1 << 5, /* Used to avoid recursion */
-  CLUTTER_TEXTURE_IN_CLONE_PAINT = 1 << 6  /* Used for safety in clones */
+  CLUTTER_TEXTURE_IN_CLONE_PAINT = 1 << 6, /* Used for safety in clones */
+  CLUTTER_STAGE_IN_RESIZE        = 1 << 7
 } ClutterPrivateFlags;
 
-typedef enum {
-  CLUTTER_PICK_NONE = 0,
-  CLUTTER_PICK_REACTIVE,
-  CLUTTER_PICK_ALL
-} ClutterPickMode;
-
 struct _ClutterInputDevice
 {
   gint          id;
-  gint32        motion_last_time;
+
+  ClutterInputDeviceType device_type;
+
   ClutterActor *pointer_grab_actor;
   ClutterActor *motion_last_actor;
 
@@ -91,7 +90,6 @@ struct _ClutterMainContext
 {
   ClutterBackend  *backend;            /* holds a pointer to the windowing 
                                           system backend */
-  ClutterStageManager *stage_manager;  /* stages */
   GQueue          *events_queue;       /* the main event queue */
 
   guint            is_initialized : 1;  
@@ -103,7 +101,6 @@ struct _ClutterMainContext
 
   ClutterPickMode  pick_mode;          /* Indicates pick render mode   */
 
-  guint            motion_frequency;   /* Motion events per second */
   gint             num_reactives;      /* Num of reactive actors */
 
   ClutterIDPool   *id_pool;            /* mapping between reused integer ids 
@@ -133,10 +130,15 @@ struct _ClutterMainContext
                                            MultiTouch */
 
   guint32 last_event_time;
+
+  gulong redraw_count;
+
+  GList *repaint_funcs;
 };
 
-#define CLUTTER_CONTEXT()	(clutter_context_get_default ())
-ClutterMainContext *clutter_context_get_default (void);
+#define CLUTTER_CONTEXT()	(_clutter_context_get_default ())
+ClutterMainContext *_clutter_context_get_default (void);
+gboolean            _clutter_context_is_initialized (void);
 PangoContext *_clutter_context_create_pango_context (ClutterMainContext *self);
 PangoContext *_clutter_context_get_pango_context    (ClutterMainContext *self);
 
@@ -174,6 +176,13 @@ ClutterStageWindow *_clutter_stage_get_window           (ClutterStage       *sta
 ClutterStageWindow *_clutter_stage_get_default_window   (void);
 void                _clutter_stage_maybe_setup_viewport (ClutterStage       *stage);
 void                _clutter_stage_maybe_relayout       (ClutterActor       *stage);
+gboolean            _clutter_stage_needs_update         (ClutterStage       *stage);
+gboolean            _clutter_stage_do_update            (ClutterStage       *stage);
+
+void     _clutter_stage_queue_event           (ClutterStage *stage,
+					       ClutterEvent *event);
+gboolean _clutter_stage_has_queued_events     (ClutterStage *stage);
+void     _clutter_stage_process_queued_events (ClutterStage *stage);
 
 /* vfuncs implemented by backend */
 GType         _clutter_backend_impl_get_type  (void);
@@ -185,6 +194,9 @@ ClutterActor *_clutter_backend_create_stage   (ClutterBackend  *backend,
                                                GError         **error);
 void          _clutter_backend_ensure_context (ClutterBackend  *backend,
                                                ClutterStage    *stage);
+gboolean      _clutter_backend_create_context (ClutterBackend  *backend,
+                                               gboolean         is_offscreen,
+                                               GError         **error);
 
 void          _clutter_backend_add_options    (ClutterBackend  *backend,
                                                GOptionGroup    *group);
@@ -196,21 +208,27 @@ void          _clutter_backend_init_events    (ClutterBackend  *backend);
 
 ClutterFeatureFlags _clutter_backend_get_features (ClutterBackend *backend);
 
-ClutterUnit   _clutter_backend_get_units_per_em   (ClutterBackend *backend);
+gfloat        _clutter_backend_get_units_per_em   (ClutterBackend       *backend,
+                                                   PangoFontDescription *font_desc);
 
 void          _clutter_feature_init (void);
 
+/* Reinjecting queued events for processing */
+void _clutter_process_event (ClutterEvent *event);
+
 /* Picking code */
-ClutterActor *_clutter_do_pick (ClutterStage   *stage,
-				 gint            x,
-				 gint            y,
-				 ClutterPickMode mode);
+ClutterActor *_clutter_do_pick (ClutterStage    *stage,
+				gint             x,
+				gint             y,
+				ClutterPickMode  mode);
+
+/* the actual redraw */
+void          _clutter_do_redraw (ClutterStage *stage);
 
 guint         _clutter_pixel_to_id (guchar pixel[4]);
 
 void          _clutter_id_to_color (guint id, ClutterColor *col);
 
-
 /* use this function as the accumulator if you have a signal with
  * a G_TYPE_BOOLEAN return value; this will stop the emission as
  * soon as one handler returns TRUE
@@ -223,12 +241,21 @@ gboolean _clutter_boolean_handled_accumulator (GSignalInvocationHint *ihint,
 void _clutter_actor_apply_modelview_transform_recursive (ClutterActor *self,
 						       ClutterActor *ancestor);
 
+void _clutter_actor_rerealize           (ClutterActor    *self,
+                                         ClutterCallback  callback,
+                                         void            *data);
+
 void _clutter_actor_set_opacity_parent (ClutterActor *self,
                                         ClutterActor *parent);
 
 void _clutter_actor_set_enable_model_view_transform (ClutterActor *self,
                                                      gboolean      enable);
 
+void _clutter_actor_set_enable_paint_unmapped (ClutterActor *self,
+                                               gboolean      enable);
+
+void _clutter_run_repaint_functions (void);
+
 G_END_DECLS
 
 #endif /* _HAVE_CLUTTER_PRIVATE_H */
