@@ -59,6 +59,7 @@
 #include "clutter-color.h"
 #include "clutter-util.h"
 #include "clutter-marshal.h"
+#include "clutter-master-clock.h"
 #include "clutter-enum-types.h"
 #include "clutter-private.h"
 #include "clutter-debug.h"
@@ -86,8 +87,9 @@ struct _ClutterStagePrivate
   gchar              *title;
   ClutterActor       *key_focused_actor;
 
-  guint               update_idle;	       /* repaint idler id */
+  GQueue             *event_queue;
 
+  guint redraw_pending    : 1;
   guint is_fullscreen     : 1;
   guint is_offscreen      : 1;
   guint is_cursor_visible : 1;
@@ -100,7 +102,7 @@ enum
   PROP_0,
 
   PROP_COLOR,
-  PROP_FULLSCREEN,
+  PROP_FULLSCREEN_SET,
   PROP_OFFSCREEN,
   PROP_CURSOR_VISIBLE,
   PROP_PERSPECTIVE,
@@ -126,13 +128,14 @@ static const ClutterColor default_stage_color = { 255, 255, 255, 255 };
 
 static void
 clutter_stage_get_preferred_width (ClutterActor *self,
-                                   ClutterUnit   for_height,
-                                   ClutterUnit  *min_width_p,
-                                   ClutterUnit  *natural_width_p)
+                                   gfloat        for_height,
+                                   gfloat       *min_width_p,
+                                   gfloat       *natural_width_p)
 {
   ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
 
-  g_assert (priv->impl != NULL);
+  if (priv->impl == NULL)
+    return;
 
   CLUTTER_ACTOR_GET_CLASS (priv->impl)->get_preferred_width (priv->impl,
                                                              for_height,
@@ -142,13 +145,14 @@ clutter_stage_get_preferred_width (ClutterActor *self,
 
 static void
 clutter_stage_get_preferred_height (ClutterActor *self,
-                                    ClutterUnit   for_width,
-                                    ClutterUnit  *min_height_p,
-                                    ClutterUnit  *natural_height_p)
+                                    gfloat        for_width,
+                                    gfloat       *min_height_p,
+                                    gfloat       *natural_height_p)
 {
   ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
 
-  g_assert (priv->impl != NULL);
+  if (priv->impl == NULL)
+    return;
 
   CLUTTER_ACTOR_GET_CLASS (priv->impl)->get_preferred_height (priv->impl,
                                                               for_width,
@@ -156,42 +160,47 @@ clutter_stage_get_preferred_height (ClutterActor *self,
                                                               natural_height_p);
 }
 static void
-clutter_stage_allocate (ClutterActor          *self,
-                        const ClutterActorBox *box,
-                        gboolean               origin_changed)
+clutter_stage_allocate (ClutterActor           *self,
+                        const ClutterActorBox  *box,
+                        ClutterAllocationFlags  flags)
 {
   ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
+  gboolean origin_changed;
 
-  g_assert (priv->impl != NULL);
+  origin_changed = (flags & CLUTTER_ABSOLUTE_ORIGIN_CHANGED) ? TRUE : FALSE;
+
+  if (priv->impl == NULL)
+    return;
 
   /* if the stage is fixed size (for instance, it's using a frame-buffer)
    * then we simply ignore any allocation request and override the
    * allocation chain.
    */
-  if (G_LIKELY (!clutter_feature_available (CLUTTER_FEATURE_STAGE_STATIC)))
+  if ((!clutter_feature_available (CLUTTER_FEATURE_STAGE_STATIC)))
     {
       ClutterActorClass *klass;
 
-      CLUTTER_NOTE (ACTOR, "Following allocation to %dx%d (origin %s)",
-                    CLUTTER_UNITS_TO_DEVICE (box->x2 - box->x1),
-                    CLUTTER_UNITS_TO_DEVICE (box->y2 - box->y1),
+      CLUTTER_NOTE (LAYOUT,
+                    "Following allocation to %dx%d (origin %s)",
+                    (int) (box->x2 - box->x1),
+                    (int) (box->y2 - box->y1),
                     origin_changed ? "changed" : "not changed");
 
       klass = CLUTTER_ACTOR_CLASS (clutter_stage_parent_class);
-      klass->allocate (self, box, origin_changed);
+      klass->allocate (self, box, flags);
 
       klass = CLUTTER_ACTOR_GET_CLASS (priv->impl);
-      klass->allocate (priv->impl, box, origin_changed);
+      klass->allocate (priv->impl, box, flags);
     }
   else
     {
       ClutterActorBox override = { 0, };
       ClutterActorClass *klass;
-      ClutterUnit natural_width, natural_height;
+      gfloat natural_width, natural_height;
 
       /* propagate the allocation */
       klass = CLUTTER_ACTOR_GET_CLASS (priv->impl);
-      klass->allocate (self, box, origin_changed);
+      klass->allocate (self, box, flags);
 
       /* get the preferred size from the backend */
       clutter_actor_get_preferred_size (priv->impl,
@@ -203,9 +212,18 @@ clutter_stage_allocate (ClutterActor          *self,
       override.x2 = natural_width;
       override.y2 = natural_height;
 
+      CLUTTER_NOTE (LAYOUT,
+                    "Overrigin original allocation of %dx%d "
+                    "with %dx%d (origin %s)",
+                    (int) (box->x2 - box->x1),
+                    (int) (box->y2 - box->y1),
+                    (int) (override.x2),
+                    (int) (override.y2),
+                    origin_changed ? "changed" : "not changed");
+
       /* and store the overridden allocation */
       klass = CLUTTER_ACTOR_CLASS (clutter_stage_parent_class);
-      klass->allocate (self, &override, origin_changed);
+      klass->allocate (self, &override, flags);
     }
 }
 
@@ -222,7 +240,9 @@ clutter_stage_paint (ClutterActor *self)
                            priv->color.green,
                            priv->color.blue,
                            priv->color.alpha);
-  cogl_clear (&stage_color);
+  cogl_clear (&stage_color,
+	      COGL_BUFFER_BIT_COLOR |
+	      COGL_BUFFER_BIT_DEPTH);
 
   if (priv->use_fog)
     {
@@ -255,7 +275,7 @@ clutter_stage_pick (ClutterActor       *self,
    * Chain up to the groups paint howerer so our children get picked
    * - clutter_group_pick
    */
-  CLUTTER_ACTOR_CLASS (clutter_stage_parent_class)->paint (self);
+  CLUTTER_ACTOR_CLASS (clutter_stage_parent_class)->pick (self, color);
 }
 
 static void
@@ -263,15 +283,14 @@ clutter_stage_realize (ClutterActor *self)
 {
   ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
 
-  CLUTTER_ACTOR_SET_FLAGS (self, CLUTTER_ACTOR_REALIZED);
-
   /* Make sure the viewport and projection matrix are valid for the
-     first paint (which will likely occur before the ConfigureNotify
-     is received) */
+   * first paint (which will likely occur before the ConfigureNotify
+   * is received)
+   */
   CLUTTER_SET_PRIVATE_FLAGS (self, CLUTTER_ACTOR_SYNC_MATRICES);
 
   g_assert (priv->impl != NULL);
-  CLUTTER_ACTOR_GET_CLASS (priv->impl)->realize (priv->impl);
+  clutter_actor_realize (priv->impl);
 
   /* ensure that the stage is using the context if the
    * realization sequence was successful
@@ -287,12 +306,9 @@ clutter_stage_unrealize (ClutterActor *self)
 {
   ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
 
-  /* unset the flag */
-  CLUTTER_ACTOR_UNSET_FLAGS (self, CLUTTER_ACTOR_REALIZED);
-
   /* and then unrealize the implementation */
   g_assert (priv->impl != NULL);
-  CLUTTER_ACTOR_GET_CLASS (priv->impl)->unrealize (priv->impl);
+  clutter_actor_unrealize (priv->impl);
 
   clutter_stage_ensure_current (CLUTTER_STAGE (self));
 }
@@ -301,33 +317,60 @@ static void
 clutter_stage_show (ClutterActor *self)
 {
   ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
-
-  g_assert (priv->impl != NULL);
-
-  if (!CLUTTER_ACTOR_IS_REALIZED (priv->impl))
-    clutter_actor_realize (priv->impl);
-
-  clutter_actor_show (priv->impl);
+  ClutterStageWindow *impl;
 
   CLUTTER_ACTOR_CLASS (clutter_stage_parent_class)->show (self);
+
+  g_assert (priv->impl != NULL);
+  impl = CLUTTER_STAGE_WINDOW (priv->impl);
+  CLUTTER_STAGE_WINDOW_GET_IFACE (impl)->show (impl, TRUE);
 }
 
 static void
 clutter_stage_hide (ClutterActor *self)
 {
   ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
+  ClutterStageWindow *impl;
 
   g_assert (priv->impl != NULL);
-  clutter_actor_hide (priv->impl);
+  impl = CLUTTER_STAGE_WINDOW (priv->impl);
+  CLUTTER_STAGE_WINDOW_GET_IFACE (impl)->hide (impl);
 
   CLUTTER_ACTOR_CLASS (clutter_stage_parent_class)->hide (self);
 }
 
+static void
+clutter_stage_emit_key_focus_event (ClutterStage *stage,
+                                    gboolean      focus_in)
+{
+  ClutterStagePrivate *priv = stage->priv;
+
+  if (priv->key_focused_actor == NULL)
+    return;
+
+  if (focus_in)
+    g_signal_emit_by_name (priv->key_focused_actor, "key-focus-in");
+  else
+    g_signal_emit_by_name (priv->key_focused_actor, "key-focus-out");
+}
+
+static void
+clutter_stage_real_activate (ClutterStage *stage)
+{
+  clutter_stage_emit_key_focus_event (stage, TRUE);
+}
+
+static void
+clutter_stage_real_deactivate (ClutterStage *stage)
+{
+  clutter_stage_emit_key_focus_event (stage, FALSE);
+}
+
 static void
 clutter_stage_real_fullscreen (ClutterStage *stage)
 {
   ClutterStagePrivate *priv = stage->priv;
-  ClutterUnit natural_width, natural_height;
+  gfloat natural_width, natural_height;
   ClutterActorBox box;
 
   /* we need to force an allocation here because the size
@@ -346,32 +389,163 @@ clutter_stage_real_fullscreen (ClutterStage *stage)
   box.x2 = natural_width;
   box.y2 = natural_height;
 
-  clutter_actor_allocate (CLUTTER_ACTOR (stage), &box, FALSE);
+  clutter_actor_allocate (CLUTTER_ACTOR (stage),
+                          &box,
+                          CLUTTER_ALLOCATION_NONE);
 }
 
-static gboolean
-redraw_update_idle (gpointer user_data)
+void
+_clutter_stage_queue_event (ClutterStage *stage,
+			    ClutterEvent *event)
 {
-  ClutterStage *stage = user_data;
-  ClutterStagePrivate *priv = stage->priv;
+  ClutterStagePrivate *priv;
+  gboolean first_event;
 
-  /* clutter_redraw() will also call maybe_relayout(), but since a relayout can
-   * queue a redraw, we want to do the relayout before we clear the update_idle
-   * to avoid painting the stage twice. Calling maybe_relayout() twice in a row
-   * is cheap because of caching of requested and allocated size.
+  g_return_if_fail (CLUTTER_IS_STAGE (stage));
+
+  priv = stage->priv;
+
+  first_event = priv->event_queue->length == 0;
+
+  g_queue_push_tail (priv->event_queue,
+		     clutter_event_copy (event));
+
+  if (first_event)
+    {
+      ClutterMasterClock *master_clock = _clutter_master_clock_get_default ();
+      _clutter_master_clock_start_running (master_clock);
+    }
+}
+
+gboolean
+_clutter_stage_has_queued_events (ClutterStage *stage)
+{
+  ClutterStagePrivate *priv;
+
+  g_return_val_if_fail (CLUTTER_IS_STAGE (stage), FALSE);
+
+  priv = stage->priv;
+
+  return priv->event_queue->length > 0;
+}
+
+void
+_clutter_stage_process_queued_events (ClutterStage *stage)
+{
+  ClutterStagePrivate *priv;
+  GList *events, *l;;
+
+  g_return_if_fail (CLUTTER_IS_STAGE (stage));
+
+  priv = stage->priv;
+
+  if (priv->event_queue->length == 0)
+    return;
+
+  /* In case the stage gets destroyed during event processing */
+  g_object_ref (stage);
+
+  /* Steal events before starting processing to avoid reentrancy
+   * issues */
+  events = priv->event_queue->head;
+  priv->event_queue->head =  NULL;
+  priv->event_queue->tail = NULL;
+  priv->event_queue->length = 0;
+
+  for (l = events; l; l = l->next)
+    {
+      ClutterEvent *event;
+      ClutterEvent *next_event;
+
+      event = l->data;
+      next_event = l->next ? l->next->data : NULL;
+
+      /* Skip consecutive motion events */
+      if (next_event &&
+	  event->type == CLUTTER_MOTION &&
+	  (next_event->type == CLUTTER_MOTION ||
+	   next_event->type == CLUTTER_LEAVE))
+	{
+	  CLUTTER_NOTE (EVENT,
+			"Omitting motion event at %.2f, %.2f",
+			event->motion.x, event->motion.y);
+	  goto next_event;
+	}
+
+      _clutter_process_event (event);
+
+    next_event:
+      clutter_event_free (event);
+    }
+
+  g_list_free (events);
+
+  g_object_unref (stage);
+}
+
+/**
+ * _clutter_stage_needs_update:
+ * @stage: A #ClutterStage
+ *
+ * Determines if _clutter_stage_do_update() needs to be called.
+ *
+ * Return value: %TRUE if the stage need layout or painting
+ */
+gboolean
+_clutter_stage_needs_update (ClutterStage *stage)
+{
+  ClutterStagePrivate *priv;
+
+  g_return_val_if_fail (CLUTTER_IS_STAGE (stage), FALSE);
+
+  priv = stage->priv;
+
+  return priv->redraw_pending;
+}
+
+/**
+ * _clutter_stage_do_update:
+ * @stage: A #ClutterStage
+ *
+ * Handles per-frame layout and repaint for the stage.
+ *
+ * Return value: %TRUE if the stage was updated
+ */
+gboolean
+_clutter_stage_do_update (ClutterStage *stage)
+{
+  ClutterStagePrivate *priv;
+
+  g_return_val_if_fail (CLUTTER_IS_STAGE (stage), FALSE);
+
+  priv = stage->priv;
+
+  if (!priv->redraw_pending)
+    return FALSE;
+
+  /* clutter_do_redraw() will also call maybe_relayout(), but since a relayout
+   * can queue a redraw, we want to do the relayout before we clear the
+   * update_idle to avoid painting the stage twice. Calling maybe_relayout()
+   * twice in a row is cheap because of caching of requested and allocated
+   * size.
    */
   _clutter_stage_maybe_relayout (CLUTTER_ACTOR (stage));
 
-  if (priv->update_idle)
+  CLUTTER_NOTE (PAINT, "redrawing via idle for stage[%p]", stage);
+  _clutter_do_redraw (stage);
+
+  /* reset the guard, so that new redraws are possible */
+  priv->redraw_pending = FALSE;
+
+  if (CLUTTER_CONTEXT ()->redraw_count > 0)
     {
-      g_source_remove (priv->update_idle);
-      priv->update_idle = 0;
+      CLUTTER_NOTE (SCHEDULER, "Queued %lu redraws during the last cycle",
+                    CLUTTER_CONTEXT ()->redraw_count);
+
+      CLUTTER_CONTEXT ()->redraw_count = 0;
     }
 
-  CLUTTER_NOTE (MULTISTAGE, "redrawing via idle for stage:%p", stage);
-  clutter_redraw (stage);
-
-  return FALSE;
+  return TRUE;
 }
 
 static void
@@ -381,17 +555,27 @@ clutter_stage_real_queue_redraw (ClutterActor *actor,
   ClutterStage *stage = CLUTTER_STAGE (actor);
   ClutterStagePrivate *priv = stage->priv;
 
-  if (priv->update_idle == 0)
-    {
-      CLUTTER_TIMESTAMP (SCHEDULER, "Adding idle source for stage: %p", stage);
+  CLUTTER_NOTE (PAINT, "Redraw request number %lu",
+                CLUTTER_CONTEXT ()->redraw_count + 1);
 
-      /* FIXME: weak_ref self in case we dissapear before paint? */
-      priv->update_idle =
-        clutter_threads_add_idle_full (CLUTTER_PRIORITY_REDRAW,
-                                       redraw_update_idle,
-                                       stage,
-                                       NULL);
+  if (!priv->redraw_pending)
+    {
+      ClutterMasterClock *master_clock;
+
+      priv->redraw_pending = TRUE;
+
+      master_clock = _clutter_master_clock_get_default ();
+      _clutter_master_clock_start_running (master_clock);
     }
+  else
+    CLUTTER_CONTEXT ()->redraw_count += 1;
+}
+
+static void
+set_offscreen_while_unrealized (ClutterActor *actor,
+                                void         *data)
+{
+  CLUTTER_STAGE (actor)->priv->is_offscreen = GPOINTER_TO_INT (data);
 }
 
 static void
@@ -415,31 +599,26 @@ clutter_stage_set_property (GObject      *object,
       break;
 
     case PROP_OFFSCREEN:
-      if (priv->is_offscreen == g_value_get_boolean (value))
-	return;
+      {
+        gboolean was_showing;
 
-      if (CLUTTER_ACTOR_IS_REALIZED (actor))
-        {
-          /* Backend needs to check this prop and handle accordingly
-           * in realise.
-           * FIXME: More 'obvious' implementation needed?
-          */
-          clutter_actor_unrealize (actor);
-          priv->is_offscreen = g_value_get_boolean (value);
-          clutter_actor_realize (actor);
+        if (priv->is_offscreen == g_value_get_boolean (value))
+          return;
 
-	  if (!CLUTTER_ACTOR_IS_REALIZED (actor))
-	    priv->is_offscreen = ~g_value_get_boolean (value);
-        }
-      else
-        priv->is_offscreen = g_value_get_boolean (value);
-      break;
+        was_showing = CLUTTER_ACTOR_IS_VISIBLE (actor);
 
-    case PROP_FULLSCREEN:
-      if (g_value_get_boolean (value))
-        clutter_stage_fullscreen (stage);
-      else
-        clutter_stage_unfullscreen (stage);
+        /* Backend needs to check this prop and handle accordingly
+         * in realise.
+         * FIXME: More 'obvious' implementation needed?
+         */
+        _clutter_actor_rerealize (actor,
+                                  set_offscreen_while_unrealized,
+                                  GINT_TO_POINTER (g_value_get_boolean (value)));
+
+        if (was_showing &&
+            !CLUTTER_ACTOR_IS_REALIZED (actor))
+          priv->is_offscreen = ~g_value_get_boolean (value);
+      }
       break;
 
     case PROP_CURSOR_VISIBLE:
@@ -493,7 +672,7 @@ clutter_stage_get_property (GObject    *gobject,
       g_value_set_boolean (value, priv->is_offscreen);
       break;
 
-    case PROP_FULLSCREEN:
+    case PROP_FULLSCREEN_SET:
       g_value_set_boolean (value, priv->is_fullscreen);
       break;
 
@@ -534,20 +713,15 @@ clutter_stage_dispose (GObject *object)
   ClutterStagePrivate *priv = stage->priv;
   ClutterStageManager *stage_manager = clutter_stage_manager_get_default ();
 
-  clutter_actor_unrealize (CLUTTER_ACTOR (object));
-
-  if (priv->update_idle)
-    {
-      g_source_remove (priv->update_idle);
-      priv->update_idle = 0;
-    }
+  clutter_actor_hide (CLUTTER_ACTOR (object));
 
   _clutter_stage_manager_remove_stage (stage_manager, stage);
 
   if (priv->impl)
     {
-      CLUTTER_NOTE (MISC, "Disposing of the stage implementation");
-      g_object_unref (priv->impl);
+      CLUTTER_NOTE (BACKEND, "Disposing of the stage implementation");
+      clutter_actor_hide (priv->impl);
+      clutter_actor_destroy (priv->impl);
       priv->impl = NULL;
     }
 
@@ -558,9 +732,13 @@ static void
 clutter_stage_finalize (GObject *object)
 {
   ClutterStage *stage = CLUTTER_STAGE (object);
+  ClutterStagePrivate *priv = stage->priv;
+
+  g_queue_foreach (priv->event_queue, (GFunc)clutter_event_free, NULL);
+  g_queue_free (priv->event_queue);
 
   g_free (stage->priv->title);
-  
+
   G_OBJECT_CLASS (clutter_stage_parent_class)->finalize (object);
 }
 
@@ -592,38 +770,51 @@ clutter_stage_class_init (ClutterStageClass *klass)
    * ClutterStage:fullscreen:
    *
    * Whether the stage should be fullscreen or not.
+   *
+   * This property is set by calling clutter_stage_set_fullscreen()
+   * but since the actual implementation is delegated to the backend
+   * you should connect to the notify::fullscreen-set signal in order
+   * to get notification if the fullscreen state has been successfully
+   * achieved.
+   *
+   * Since: 1.0
    */
-  g_object_class_install_property
-    (gobject_class, PROP_FULLSCREEN,
-     g_param_spec_boolean ("fullscreen",
-			   "Fullscreen",
-			   "Whether the main stage is fullscreen",
-			   FALSE,
-			   G_PARAM_CONSTRUCT | CLUTTER_PARAM_READWRITE));
+  pspec = g_param_spec_boolean ("fullscreen-set",
+                                "Fullscreen Set",
+                                "Whether the main stage is fullscreen",
+                                FALSE,
+                                CLUTTER_PARAM_READABLE);
+  g_object_class_install_property (gobject_class,
+                                   PROP_FULLSCREEN_SET,
+                                   pspec);
   /**
    * ClutterStage:offscreen:
    *
    * Whether the stage should be rendered in an offscreen buffer.
    */
-  g_object_class_install_property
-    (gobject_class, PROP_OFFSCREEN,
-     g_param_spec_boolean ("offscreen",
-			   "Offscreen",
-			   "Whether the main stage is renderer offscreen",
-			   FALSE,
-			   G_PARAM_CONSTRUCT | CLUTTER_PARAM_READWRITE));
+  pspec = g_param_spec_boolean ("offscreen",
+                                "Offscreen",
+                                "Whether the main stage should be "
+                                "rendered offscreen",
+                                FALSE,
+                                CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class,
+                                   PROP_OFFSCREEN,
+                                   pspec);
   /**
    * ClutterStage:cursor-visible:
    *
    * Whether the mouse pointer should be visible
    */
-  g_object_class_install_property
-    (gobject_class, PROP_CURSOR_VISIBLE,
-     g_param_spec_boolean ("cursor-visible",
-			   "Cursor Visible",
-			   "Whether the mouse pointer is visible on the main stage ",
-			   TRUE,
-			   G_PARAM_CONSTRUCT | CLUTTER_PARAM_READWRITE));
+  pspec = g_param_spec_boolean ("cursor-visible",
+                                "Cursor Visible",
+                                "Whether the mouse pointer is visible "
+                                "on the main stage",
+                                TRUE,
+                                CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class,
+                                   PROP_CURSOR_VISIBLE,
+                                   pspec);
   /**
    * ClutterStage:user-resizable:
    *
@@ -631,14 +822,15 @@ clutter_stage_class_init (ClutterStageClass *klass)
    *
    * Since: 0.4
    */
-  g_object_class_install_property
-    (gobject_class, PROP_USER_RESIZE,
-     g_param_spec_boolean ("user-resizable",
-			   "User Resizable",
-			   "Whether the stage is able to be resized via "
-			   "user interaction",
-			   FALSE,
-			   G_PARAM_CONSTRUCT | CLUTTER_PARAM_READWRITE));
+  pspec = g_param_spec_boolean ("user-resizable",
+                                "User Resizable",
+                                "Whether the stage is able to be resized "
+                                "via user interaction",
+                                FALSE,
+                                CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class,
+                                   PROP_USER_RESIZE,
+                                   pspec);
   /**
    * ClutterStage:color:
    *
@@ -659,13 +851,14 @@ clutter_stage_class_init (ClutterStageClass *klass)
    *
    * Since: 0.8.2
    */
-  g_object_class_install_property
-    (gobject_class, PROP_PERSPECTIVE,
-     g_param_spec_boxed ("perspective",
-			 "Perspective",
-			 "Perspective projection parameters",
-			 CLUTTER_TYPE_PERSPECTIVE,
-			 CLUTTER_PARAM_READWRITE));
+  pspec = g_param_spec_boxed ("perspective",
+                              "Perspective",
+                              "Perspective projection parameters",
+                              CLUTTER_TYPE_PERSPECTIVE,
+                              CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class,
+                                   PROP_PERSPECTIVE,
+                                   pspec);
 
   /**
    * ClutterStage:title:
@@ -674,13 +867,13 @@ clutter_stage_class_init (ClutterStageClass *klass)
    *
    * Since: 0.4
    */
-  g_object_class_install_property
-    (gobject_class, PROP_TITLE,
-     g_param_spec_string ("title",
-			  "Title",
-			  "Stage Title",
-			  NULL,
-			  CLUTTER_PARAM_READWRITE));
+  pspec = g_param_spec_string ("title",
+                               "Title",
+                               "Stage Title",
+                               NULL,
+                               CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_TITLE, pspec);
+
   /**
    * ClutterStage:use-fog:
    *
@@ -690,13 +883,13 @@ clutter_stage_class_init (ClutterStageClass *klass)
    *
    * Since: 0.6
    */
-  g_object_class_install_property (gobject_class,
-                                   PROP_USE_FOG,
-                                   g_param_spec_boolean ("use-fog",
-                                                         "Use Fog",
-                                                         "Whether to enable depth cueing",
-                                                         FALSE,
-                                                         CLUTTER_PARAM_READWRITE));
+  pspec = g_param_spec_boolean ("use-fog",
+                                "Use Fog",
+                                "Whether to enable depth cueing",
+                                FALSE,
+                                CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_USE_FOG, pspec);
+
   /**
    * ClutterStage:fog:
    *
@@ -781,6 +974,8 @@ clutter_stage_class_init (ClutterStageClass *klass)
 		  G_TYPE_NONE, 0);
 
   klass->fullscreen = clutter_stage_real_fullscreen;
+  klass->activate = clutter_stage_real_activate;
+  klass->deactivate = clutter_stage_real_deactivate;
 
   g_type_class_add_private (gobject_class, sizeof (ClutterStagePrivate));
 }
@@ -811,6 +1006,11 @@ clutter_stage_init (ClutterStage *self)
   else
     g_object_ref_sink (priv->impl);
 
+  /* make sure that the implementation is considered a top level */
+  CLUTTER_SET_PRIVATE_FLAGS (priv->impl, CLUTTER_ACTOR_IS_TOPLEVEL);
+
+  priv->event_queue = g_queue_new ();
+
   priv->is_offscreen      = FALSE;
   priv->is_fullscreen     = FALSE;
   priv->is_user_resizable = FALSE;
@@ -924,6 +1124,7 @@ clutter_stage_set_perspective (ClutterStage       *stage,
 
   g_return_if_fail (CLUTTER_IS_STAGE (stage));
   g_return_if_fail (perspective != NULL);
+  g_return_if_fail (perspective->z_far - perspective->z_near != 0);
 
   priv = stage->priv;
 
@@ -953,23 +1154,36 @@ clutter_stage_get_perspective (ClutterStage       *stage,
 }
 
 /**
- * clutter_stage_fullscreen:
+ * clutter_stage_set_fullscreen:
  * @stage: a #ClutterStage
+ * @fullscreen: %TRUE to to set the stage fullscreen
  *
- * Asks to place the stage window in the fullscreen state. Note that you
- * shouldn't assume the window is definitely full screen afterward, because
- * other entities (e.g. the user or window manager) could unfullscreen it
- * again, and not all window managers honor requests to fullscreen windows.
+ * Asks to place the stage window in the fullscreen or unfullscreen
+ * states.
+ *
+ ( Note that you shouldn't assume the window is definitely full screen
+ * afterward, because other entities (e.g. the user or window manager)
+ * could unfullscreen it again, and not all window managers honor
+ * requests to fullscreen windows.
+ *
+ * If you want to receive notification of the fullscreen state you
+ * should either use the #ClutterStage::fullscreen and
+ * #ClutterStage::unfullscreen signals, or use the notify signal
+ * for the #ClutterStage:fullscreen-set property
+ *
+ * Since: 1.0
  */
 void
-clutter_stage_fullscreen (ClutterStage *stage)
+clutter_stage_set_fullscreen (ClutterStage *stage,
+                              gboolean      fullscreen)
 {
   ClutterStagePrivate *priv;
 
   g_return_if_fail (CLUTTER_IS_STAGE (stage));
 
   priv = stage->priv;
-  if (!priv->is_fullscreen)
+
+  if (priv->is_fullscreen != fullscreen)
     {
       ClutterStageWindow *impl = CLUTTER_STAGE_WINDOW (priv->impl);
       ClutterStageWindowIface *iface;
@@ -977,46 +1191,31 @@ clutter_stage_fullscreen (ClutterStage *stage)
       iface = CLUTTER_STAGE_WINDOW_GET_IFACE (impl);
 
       /* Only set if backend implements.
+       *
        * Also see clutter_stage_event() for setting priv->is_fullscreen
        * on state change event.
-      */
+       */
       if (iface->set_fullscreen)
-	iface->set_fullscreen (impl, TRUE);
+	iface->set_fullscreen (impl, fullscreen);
     }
 }
 
 /**
- * clutter_stage_unfullscreen:
+ * clutter_stage_get_fullscreen:
  * @stage: a #ClutterStage
  *
- * Asks to toggle off the fullscreen state for the stage window. Note that
- * you shouldn't assume the window is definitely not full screen afterward,
- * because other entities (e.g. the user or window manager) could fullscreen
- * it again, and not all window managers honor requests to unfullscreen
- * windows.
+ * Retrieves whether the stage is full screen or not
+ *
+ * Return value: %TRUE if the stage is full screen
+ *
+ * Since: 1.0
  */
-void
-clutter_stage_unfullscreen (ClutterStage *stage)
+gboolean
+clutter_stage_get_fullscreen (ClutterStage *stage)
 {
-  ClutterStagePrivate *priv;
+  g_return_val_if_fail (CLUTTER_IS_STAGE (stage), FALSE);
 
-  g_return_if_fail (CLUTTER_IS_STAGE (stage));
-
-  priv = stage->priv;
-  if (priv->is_fullscreen)
-    {
-      ClutterStageWindow *impl = CLUTTER_STAGE_WINDOW (priv->impl);
-      ClutterStageWindowIface *iface;
-
-      iface = CLUTTER_STAGE_WINDOW_GET_IFACE (impl);
-
-      /* Only set if backend implements.
-       * Also see clutter_stage_event() for setting priv->is_fullscreen
-       * on state change event.
-      */
-      if (iface->set_fullscreen)
-	iface->set_fullscreen (impl, FALSE);
-    }
+  return stage->priv->is_fullscreen;
 }
 
 /**
@@ -1167,10 +1366,9 @@ clutter_stage_read_pixels (ClutterStage *stage,
                            gint          height)
 {
   guchar *pixels;
-  guchar *temprow;
   GLint   viewport[4];
   gint    rowstride;
-  gint    stage_x, stage_y, stage_width, stage_height;
+  gint    stage_width, stage_height;
 
   g_return_val_if_fail (CLUTTER_IS_STAGE (stage), NULL);
 
@@ -1180,12 +1378,10 @@ clutter_stage_read_pixels (ClutterStage *stage,
   g_return_val_if_fail (x >= 0 && y >= 0, NULL);
 
   /* Force a redraw of the stage before reading back pixels */
-  clutter_redraw (stage);
   clutter_stage_ensure_current (stage);
+  clutter_actor_paint (CLUTTER_ACTOR (stage));
 
   glGetIntegerv (GL_VIEWPORT, viewport);
-  stage_x      = viewport[0];
-  stage_y      = viewport[1];
   stage_width  = viewport[2];
   stage_height = viewport[3];
 
@@ -1198,42 +1394,11 @@ clutter_stage_read_pixels (ClutterStage *stage,
   rowstride = width * 4;
 
   pixels  = g_malloc (height * rowstride);
-  temprow = g_malloc (rowstride);
 
-  /* Setup the pixel store parameters that may have been changed by
-     Cogl */
-  glPixelStorei (GL_PACK_ALIGNMENT, 4);
-#ifdef HAVE_COGL_GL
-  glPixelStorei (GL_PACK_ROW_LENGTH, 0);
-  glPixelStorei (GL_PACK_SKIP_PIXELS, 0);
-  glPixelStorei (GL_PACK_SKIP_ROWS, 0);
-#endif /* HAVE_COGL_GL */
-
-  /* The y co-ordinate should be given in OpenGL's coordinate system
-     so 0 is the bottom row */
-  y = stage_height - y - height;
-
-  glFinish ();
-
-  /* check whether we need to read into a smaller temporary buffer */
-  glReadPixels (x, y, width, height, GL_RGBA, GL_UNSIGNED_BYTE, pixels);
-
-  /* vertically flip the buffer in-place */
-  for (y = 0; y < height / 2; y++)
-    {
-      if (y != height - y - 1) /* skip center row */
-        {
-          memcpy (temprow,
-                  pixels + y * rowstride, rowstride);
-          memcpy (pixels + y * rowstride,
-                  pixels + (height - y - 1) * rowstride, rowstride);
-          memcpy (pixels + (height - y - 1) * rowstride,
-                  temprow,
-                  rowstride);
-        }
-    }
-
-  g_free (temprow);
+  cogl_read_pixels (x, y, width, height,
+                    COGL_READ_PIXELS_COLOR_BUFFER,
+                    COGL_PIXEL_FORMAT_RGBA_8888,
+                    pixels);
 
   return pixels;
 }
@@ -1241,20 +1406,26 @@ clutter_stage_read_pixels (ClutterStage *stage,
 /**
  * clutter_stage_get_actor_at_pos:
  * @stage: a #ClutterStage
+ * @pick_mode: how the scene graph should be painted
  * @x: X coordinate to check
  * @y: Y coordinate to check
  *
  * Checks the scene at the coordinates @x and @y and returns a pointer
  * to the #ClutterActor at those coordinates.
  *
- * Return value: (transfer none): the actor at the specified coordinates, if any
+ * By using @pick_mode it is possible to control which actors will be
+ * painted and thus available.
+ *
+ * Return value: (transfer none): the actor at the specified coordinates,
+ *   if any
  */
 ClutterActor *
-clutter_stage_get_actor_at_pos (ClutterStage *stage,
-                                gint          x,
-                                gint          y)
+clutter_stage_get_actor_at_pos (ClutterStage    *stage,
+                                ClutterPickMode  pick_mode,
+                                gint             x,
+                                gint             y)
 {
-  return _clutter_do_pick (stage, x, y, CLUTTER_PICK_ALL);
+  return _clutter_do_pick (stage, x, y, pick_mode);
 }
 
 /**
@@ -1305,14 +1476,14 @@ clutter_stage_event (ClutterStage *stage,
 	  priv->is_fullscreen = TRUE;
 	  g_signal_emit (stage, stage_signals[FULLSCREEN], 0);
 
-          g_object_notify (G_OBJECT (stage), "fullscreen");
+          g_object_notify (G_OBJECT (stage), "fullscreen-set");
 	}
       else
 	{
 	  priv->is_fullscreen = FALSE;
 	  g_signal_emit (stage, stage_signals[UNFULLSCREEN], 0);
 
-          g_object_notify (G_OBJECT (stage), "fullscreen");
+          g_object_notify (G_OBJECT (stage), "fullscreen-set");
 	}
     }
 
@@ -1422,15 +1593,31 @@ clutter_stage_set_key_focus (ClutterStage *stage,
 
   if (priv->key_focused_actor)
     {
+      ClutterActor *old_focused_actor;
+
+      old_focused_actor = priv->key_focused_actor;
+
+      /* set key_focused_actor to NULL before emitting the signal or someone
+       * might hide the previously focused actor in the signal handler and we'd
+       * get re-entrant call and get glib critical from g_object_weak_unref
+       */
+
       g_object_weak_unref (G_OBJECT (priv->key_focused_actor),
 			   on_key_focused_weak_notify,
 			   stage);
-      g_signal_emit_by_name (priv->key_focused_actor, "focus-out");
 
       priv->key_focused_actor = NULL;
+
+      g_signal_emit_by_name (old_focused_actor, "key-focus-out");
     }
   else
-    g_signal_emit_by_name (stage, "focus-out");
+    g_signal_emit_by_name (stage, "key-focus-out");
+
+  /* Note, if someone changes key focus in focus-out signal handler we'd be
+   * overriding the latter call below moving the focus where it was originally
+   * intended. The order of events would be:
+   *   1st focus-out, 2nd focus-out (on stage), 2nd focus-in, 1st focus-in
+   */
 
   if (actor)
     {
@@ -1439,10 +1626,10 @@ clutter_stage_set_key_focus (ClutterStage *stage,
       g_object_weak_ref (G_OBJECT (actor),
 			 on_key_focused_weak_notify,
 			 stage);
-      g_signal_emit_by_name (priv->key_focused_actor, "focus-in");
+      g_signal_emit_by_name (priv->key_focused_actor, "key-focus-in");
     }
   else
-    g_signal_emit_by_name (stage, "focus-in");
+    g_signal_emit_by_name (stage, "key-focus-in");
 }
 
 /**
@@ -1570,6 +1757,16 @@ clutter_stage_set_use_fog (ClutterStage *stage,
  *   }
  * ]|
  *
+ * Note: The fogging functions only work correctly when the visible actors use
+ *       unmultiplied alpha colors. By default Cogl will premultiply textures
+ *       and cogl_set_source_color will premultiply colors, so unless you
+ *       explicitly load your textures requesting an unmultiplied
+ *       internal_format and use cogl_material_set_color you can only use
+ *       fogging with fully opaque actors.
+ *
+ *       We can look to improve this in the future when we can depend on
+ *       fragment shaders.
+ *
  * Since: 0.6
  */
 void
@@ -1716,11 +1913,12 @@ clutter_stage_new (void)
 void
 clutter_stage_ensure_current (ClutterStage *stage)
 {
-  ClutterMainContext *ctx = clutter_context_get_default ();
+  ClutterBackend *backend;
 
   g_return_if_fail (CLUTTER_IS_STAGE (stage));
 
-  _clutter_backend_ensure_context (ctx->backend, stage);
+  backend = clutter_get_default_backend ();
+  _clutter_backend_ensure_context (backend, stage);
 }
 
 /**
@@ -1748,6 +1946,33 @@ clutter_stage_ensure_viewport (ClutterStage *stage)
   clutter_actor_queue_redraw (CLUTTER_ACTOR (stage));
 }
 
+/**
+ * clutter_stage_ensure_redraw:
+ * @stage: a #ClutterStage
+ *
+ * Ensures that @stage is redrawn
+ *
+ * This function should not be called by applications: it is
+ * used when embedding a #ClutterStage into a toolkit with
+ * another windowing system, like GTK+.
+ *
+ * Since: 1.0
+ */
+void
+clutter_stage_ensure_redraw (ClutterStage *stage)
+{
+  ClutterMasterClock *master_clock;
+  ClutterStagePrivate *priv;
+
+  g_return_if_fail (CLUTTER_IS_STAGE (stage));
+
+  priv = stage->priv;
+  priv->redraw_pending = TRUE;
+
+  master_clock = _clutter_master_clock_get_default ();
+  _clutter_master_clock_start_running (master_clock);
+}
+
 /**
  * clutter_stage_queue_redraw:
  * @stage: the #ClutterStage
