@@ -79,6 +79,22 @@ G_BEGIN_DECLS
 #define CLUTTER_STAGE_HEIGHT() \
  (clutter_actor_get_height (clutter_stage_get_default ()))
 
+/**
+ * ClutterPickMode:
+ * @CLUTTER_PICK_NONE: Do not paint any actor
+ * @CLUTTER_PICK_REACTIVE: Paint only the reactive actors
+ * @CLUTTER_PICK_ALL: Paint all actors
+ *
+ * Controls the paint cycle of the scene graph when in pick mode
+ *
+ * Since: 1.0
+ */
+typedef enum {
+  CLUTTER_PICK_NONE = 0,
+  CLUTTER_PICK_REACTIVE,
+  CLUTTER_PICK_ALL
+} ClutterPickMode;
+
 typedef struct _ClutterPerspective  ClutterPerspective;
 typedef struct _ClutterFog          ClutterFog;
 
@@ -186,12 +202,14 @@ void          clutter_stage_set_perspective   (ClutterStage       *stage,
 			                       ClutterPerspective *perspective);
 void          clutter_stage_get_perspective   (ClutterStage       *stage,
 			                       ClutterPerspective *perspective);
-void          clutter_stage_fullscreen        (ClutterStage       *stage);
-void          clutter_stage_unfullscreen      (ClutterStage       *stage);
+void          clutter_stage_set_fullscreen    (ClutterStage       *stage,
+                                               gboolean            fullscreen);
+gboolean      clutter_stage_get_fullscreen    (ClutterStage       *stage);
 void          clutter_stage_show_cursor       (ClutterStage       *stage);
 void          clutter_stage_hide_cursor       (ClutterStage       *stage);
 
 ClutterActor *clutter_stage_get_actor_at_pos  (ClutterStage       *stage,
+                                               ClutterPickMode     pick_mode,
                                                gint                x,
                                                gint                y);
 guchar *      clutter_stage_read_pixels       (ClutterStage       *stage,
@@ -225,6 +243,7 @@ void                  clutter_stage_ensure_current     (ClutterStage *stage);
 void                  clutter_stage_queue_redraw       (ClutterStage *stage);
 gboolean              clutter_stage_is_default         (ClutterStage *stage);
 void                  clutter_stage_ensure_viewport    (ClutterStage *stage);
+void                  clutter_stage_ensure_redraw      (ClutterStage *stage);
 
 /* Commodity macro */
 #define clutter_stage_add(stage,actor)                  G_STMT_START {  \
