@@ -1,7 +1,9 @@
-/* cogl-texture.h: Texture objects
- * This file is part of Clutter
+/*
+ * Cogl
  *
- * Copyright (C) 2008  Intel Corporation.
+ * An object oriented GL/GLES Abstraction/Utility Layer
+ *
+ * Copyright (C) 2007,2008,2009 Intel Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -14,7 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #if !defined(__COGL_H_INSIDE__) && !defined(CLUTTER_COMPILATION)
@@ -24,10 +28,10 @@
 #ifndef __COGL_TEXTURE_H__
 #define __COGL_TEXTURE_H__
 
-G_BEGIN_DECLS
-
 #include <cogl/cogl-types.h>
 
+G_BEGIN_DECLS
+
 /**
  * SECTION:cogl-texture
  * @short_description: Fuctions for creating and manipulating textures
@@ -37,12 +41,12 @@ G_BEGIN_DECLS
  * loading and manipulating textures.
  */
 
+#define COGL_TEXTURE_MAX_WASTE  127
+
 /**
  * cogl_texture_new_with_size:
  * @width: width of texture in pixels.
  * @height: height of texture in pixels.
- * @max_waste: maximum extra horizontal and|or vertical margin pixels
- *    to make the texture fit GPU limitations
  * @flags: Optional flags for the texture, or %COGL_TEXTURE_NONE
  * @internal_format: the #CoglPixelFormat to use for the GPU storage of the
  *    texture.
@@ -56,18 +60,21 @@ G_BEGIN_DECLS
  */
 CoglHandle      cogl_texture_new_with_size    (guint            width,
                                                guint            height,
-                                               gint             max_waste,
                                                CoglTextureFlags flags,
                                                CoglPixelFormat  internal_format);
 
 /**
  * cogl_texture_new_from_file:
  * @filename: the file to load
- * @max_waste: maximum extra horizontal and|or vertical margin pixels
- *    to make the texture fit GPU limitations
  * @flags: Optional flags for the texture, or %COGL_TEXTURE_NONE
  * @internal_format: the #CoglPixelFormat to use for the GPU storage of the
- *    texture
+ *    texture. If COGL_PIXEL_FORMAT_ANY is given then a premultiplied
+ *    format similar to the format of the source data will be used. The
+ *    default blending equations of Cogl expect premultiplied color data;
+ *    the main use of passing a non-premultiplied format here is if you
+ *    have non-premultiplied source data and are going to adjust the blend
+ *    mode (see cogl_material_set_blend()) or use the data for something
+ *    other than straight blending.
  * @error: return location for a #GError or %NULL
  *
  * Creates a COGL texture from an image file.
@@ -78,7 +85,6 @@ CoglHandle      cogl_texture_new_with_size    (guint            width,
  * Since: 0.8
  */
 CoglHandle      cogl_texture_new_from_file    (const gchar       *filename,
-                                               gint               max_waste,
                                                CoglTextureFlags   flags,
                                                CoglPixelFormat    internal_format,
                                                GError           **error);
@@ -87,12 +93,16 @@ CoglHandle      cogl_texture_new_from_file    (const gchar       *filename,
  * cogl_texture_new_from_data:
  * @width: width of texture in pixels
  * @height: height of texture in pixels
- * @max_waste: maximum extra horizontal and|or vertical margin pixels
- *    to make the texture fit GPU limitations
  * @flags: Optional flags for the texture, or %COGL_TEXTURE_NONE
  * @format: the #CoglPixelFormat the buffer is stored in in RAM
  * @internal_format: the #CoglPixelFormat that will be used for storing
- *    the buffer on the GPU
+ *    the buffer on the GPU. If COGL_PIXEL_FORMAT_ANY is given then a
+ *    premultiplied format similar to the format of the source data will
+ *    be used. The default blending equations of Cogl expect premultiplied
+ *    color data; the main use of passing a non-premultiplied format here
+ *    is if you have non-premultiplied source data and are going to adjust
+ *    the blend mode (see cogl_material_set_blend()) or use the data for
+ *    something other than straight blending.
  * @rowstride: the memory offset in bytes between the starts of
  *    scanlines in @data
  * @data: pointer the memory region where the source buffer resides
@@ -106,7 +116,6 @@ CoglHandle      cogl_texture_new_from_file    (const gchar       *filename,
  */
 CoglHandle      cogl_texture_new_from_data    (guint             width,
                                                guint             height,
-                                               gint              max_waste,
                                                CoglTextureFlags  flags,
                                                CoglPixelFormat   format,
                                                CoglPixelFormat   internal_format,
@@ -132,34 +141,31 @@ CoglHandle      cogl_texture_new_from_data    (guint             width,
  *
  * Since: 0.8
  */
-CoglHandle      cogl_texture_new_from_foreign (GLuint              gl_handle,
-                                               GLenum              gl_target,
-                                               GLuint              width,
-                                               GLuint              height,
-                                               GLuint              x_pot_waste,
-                                               GLuint              y_pot_waste,
-                                               CoglPixelFormat     format);
+CoglHandle      cogl_texture_new_from_foreign (GLuint          gl_handle,
+                                               GLenum          gl_target,
+                                               GLuint          width,
+                                               GLuint          height,
+                                               GLuint          x_pot_waste,
+                                               GLuint          y_pot_waste,
+                                               CoglPixelFormat format);
 
 /**
  * cogl_texture_new_from_bitmap:
- * @bitmap: a #CoglBitmap
- * @max_waste: maximum extra horizontal and|or vertical margin pixels
- *    to make the texture fit GPU limitations
+ * @bmp_handle: A CoglBitmap handle
  * @flags: Optional flags for the texture, or %COGL_TEXTURE_NONE
  * @internal_format: the #CoglPixelFormat to use for the GPU storage of the
  * texture
  *
- * Creates a COGL texture from a #CoglBitmap.
+ * Creates a COGL texture from a CoglBitmap.
  *
  * Return value: a #CoglHandle to the newly created texture or
  *   %COGL_INVALID_HANDLE on failure
  *
  * Since: 1.0
  */
-CoglHandle      cogl_texture_new_from_bitmap (CoglBitmap       *bitmap,
-                                              gint              max_waste,
-                                              CoglTextureFlags  flags,
-                                              CoglPixelFormat   internal_format);
+CoglHandle      cogl_texture_new_from_bitmap (CoglHandle       bmp_handle,
+                                              CoglTextureFlags flags,
+                                              CoglPixelFormat  internal_format);
 
 /**
  * cogl_is_texture:
@@ -224,26 +230,6 @@ guint           cogl_texture_get_rowstride    (CoglHandle          handle);
  */
 gint            cogl_texture_get_max_waste    (CoglHandle          handle);
 
-/**
- * cogl_texture_get_min_filter:
- * @handle: a #CoglHandle for a texture.
- *
- * Query the currently set downscaling filter for a cogl texture.
- *
- * Returns: the current downscaling filter for a cogl texture.
- */
-COGLenum        cogl_texture_get_min_filter   (CoglHandle          handle);
-
-/**
- * cogl_texture_get_mag_filter:
- * @handle: a #CoglHandle for a texture.
- *
- * Query the currently set downscaling filter for a cogl texture.
- *
- * Returns: the current downscaling filter for a cogl texture.
- */
-COGLenum        cogl_texture_get_mag_filter   (CoglHandle          handle);
-
 /**
  * cogl_texture_is_sliced:
  * @handle: a #CoglHandle for a texture.
@@ -293,20 +279,6 @@ gint            cogl_texture_get_data         (CoglHandle          handle,
                                                guint               rowstride,
                                                guchar             *data);
 
-/**
- * cogl_texture_set_filters:
- * @handle: a #CoglHandle.
- * @min_filter: the filter used when scaling the texture down.
- * @mag_filter: the filter used when magnifying the texture.
- *
- * Changes the decimation and interpolation filters used when the texture is
- * drawn at other scales than 100%.
- */
-void            cogl_texture_set_filters      (CoglHandle          handle,
-                                               COGLenum            min_filter,
-                                               COGLenum            mag_filter);
-
-
 /**
  * cogl_texture_set_region:
  * @handle: a #CoglHandle.
@@ -359,45 +331,6 @@ CoglHandle      cogl_texture_ref              (CoglHandle          handle);
  */
 void            cogl_texture_unref            (CoglHandle          handle);
 
-/**
- * cogl_bitmap_new_from_file:
- * @filename: the file to load.
- * @error: a #GError or %NULL.
- *
- * Load an image file from disk. This function can be safely called from
- * within a thread.
- *
- * Returns: A #CoglBitmap to the new loaded image data, or %NULL if loading
- * the image failed.
- *
- * Since: 1.0
- */
-CoglBitmap *    cogl_bitmap_new_from_file     (const gchar    *filename,
-                                               GError        **error);
-
-/**
- * cogl_bitmap_get_size_from_file:
- * @filename: the file to check
- * @width: return location for the bitmap width
- * @height: return location for the bitmap height
- *
- * Parses an image file enough to extract the width and height
- * of the bitmap.
- *
- * Since: 1.0
- */
-gboolean        cogl_bitmap_get_size_from_file (const gchar   *filename,
-                                                gint          *width,
-                                                gint          *height);
-
-/**
- * cogl_bitmap_free:
- * @bmp: a #CoglBitmap.
- *
- * Frees a #CoglBitmap.
- */
-void            cogl_bitmap_free              (CoglBitmap     *bmp);
-
 /**
  * cogl_rectangle_with_texture_coords:
  * @x1: x coordinate upper left on screen.
@@ -477,6 +410,27 @@ void cogl_rectangle_with_multitexture_coords (float        x1,
 void  cogl_rectangles_with_texture_coords (const float *verts,
                                            guint        n_rects);
 
+/**
+ * cogl_rectangles:
+ * @verts: an array of vertices
+ * @n_rects: number of rectangles to draw
+ *
+ * Draws a series of rectangles in the same way that
+ * cogl_rectangle() does. In some situations it can give a
+ * significant performance boost to use this function rather than
+ * calling cogl_rectangle() separately for each rectangle.
+ *
+ * @verts should point to an array of #float<!-- -->s with
+ * @n_rects * 4 elements. Each group of 4 values corresponds to the
+ * parameters x1, y1, x2, and y2, and have the same
+ * meaning as in cogl_rectangle().
+ *
+ * Since: 1.0
+ */
+void cogl_rectangles (const float *verts,
+                      guint        n_rects);
+
+
 /**
  * cogl_polygon:
  * @vertices: An array of #CoglTextureVertex structs
