@@ -1,11 +1,9 @@
 /*
- * Clutter COGL
+ * Cogl
  *
- * A basic GL/GLES Abstraction/Utility Layer
+ * An object oriented GL/GLES Abstraction/Utility Layer
  *
- * Authored By Matthew Allum  <mallum@openedhand.com>
- *
- * Copyright (C) 2007 OpenedHand
+ * Copyright (C) 2007,2008,2009 Intel Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -29,10 +27,21 @@
 
 #include "cogl.h"
 #include "cogl-internal.h"
-#include "cogl-bitmap.h"
+#include "cogl-bitmap-private.h"
 
 #include <string.h>
 
+static void _cogl_bitmap_free (CoglBitmap *bmp);
+
+COGL_HANDLE_DEFINE (Bitmap, bitmap);
+
+static void
+_cogl_bitmap_free (CoglBitmap *bmp)
+{
+  g_free (bmp->data);
+  g_free (bmp);
+}
+
 gint
 _cogl_get_format_bpp (CoglPixelFormat format)
 {
@@ -47,7 +56,7 @@ _cogl_get_format_bpp (CoglPixelFormat format)
     2, /* YUV      */
     1  /* G_8      */
   };
-  
+
   return bpp_lut [format & COGL_UNORDERED_MASK];
 }
 
@@ -59,7 +68,7 @@ _cogl_bitmap_convert_and_premult (const CoglBitmap *bmp,
   CoglBitmap  tmp_bmp = *bmp;
   CoglBitmap  new_bmp = *bmp;
   gboolean    new_bmp_owner = FALSE;
-  
+
   /* Is base format different (not considering premult status)? */
   if ((bmp->format & COGL_UNPREMULT_MASK) !=
       (dst_format & COGL_UNPREMULT_MASK))
@@ -71,15 +80,15 @@ _cogl_bitmap_convert_and_premult (const CoglBitmap *bmp,
 	  if (!_cogl_bitmap_fallback_convert (&new_bmp, &tmp_bmp, dst_format))
 	    return FALSE;
 	}
-  
+
       /* Update bitmap with new data */
       new_bmp = tmp_bmp;
       new_bmp_owner = TRUE;
     }
-  
+
   /* Do we need to unpremultiply */
-  if ((bmp->format & COGL_PREMULT_BIT) == 0 &&
-      (dst_format & COGL_PREMULT_BIT) > 0)
+  if ((bmp->format & COGL_PREMULT_BIT) > 0 &&
+      (dst_format & COGL_PREMULT_BIT) == 0)
     {
       /* Try unpremultiplying using imaging library */
       if (!_cogl_bitmap_unpremult (&new_bmp, &tmp_bmp))
@@ -89,33 +98,47 @@ _cogl_bitmap_convert_and_premult (const CoglBitmap *bmp,
 	    {
 	      if (new_bmp_owner)
 		g_free (new_bmp.data);
-	      
+
 	      return FALSE;
 	    }
 	}
-      
+
       /* Update bitmap with new data */
       if (new_bmp_owner)
 	g_free (new_bmp.data);
-      
+
       new_bmp = tmp_bmp;
       new_bmp_owner = TRUE;
     }
-  
+
   /* Do we need to premultiply */
-  if ((bmp->format & COGL_PREMULT_BIT) > 0 &&
-      (dst_format & COGL_PREMULT_BIT) == 0)
+  if ((bmp->format & COGL_PREMULT_BIT) == 0 &&
+      (dst_format & COGL_PREMULT_BIT) > 0)
     {
-      /* FIXME: implement premultiplication */
+      /* Try premultiplying using imaging library */
+      if (!_cogl_bitmap_premult (&new_bmp, &tmp_bmp))
+	{
+	  /* ... or try fallback */
+	  if (!_cogl_bitmap_fallback_premult (&new_bmp, &tmp_bmp))
+	    {
+	      if (new_bmp_owner)
+		g_free (new_bmp.data);
+
+	      return FALSE;
+	    }
+	}
+
+      /* Update bitmap with new data */
       if (new_bmp_owner)
 	g_free (new_bmp.data);
-      
-      return FALSE;
+
+      new_bmp = tmp_bmp;
+      new_bmp_owner = TRUE;
     }
-  
+
   /* Output new bitmap info */
   *dst_bmp = new_bmp;
-  
+
   return TRUE;
 }
 
@@ -133,14 +156,14 @@ _cogl_bitmap_copy_subregion (CoglBitmap *src,
   guchar *dstdata;
   gint    bpp;
   gint    line;
-  
+
   /* Intended only for fast copies when format is equal! */
   g_assert (src->format == dst->format);
   bpp = _cogl_get_format_bpp (src->format);
-  
+
   srcdata = src->data + src_y * src->rowstride + src_x * bpp;
   dstdata = dst->data + dst_y * dst->rowstride + dst_x * bpp;
-  
+
   for (line=0; line<height; ++line)
     {
       memcpy (dstdata, srcdata, width * bpp);
@@ -157,12 +180,13 @@ cogl_bitmap_get_size_from_file (const gchar *filename,
   return _cogl_bitmap_get_size_from_file (filename, width, height);
 }
 
-CoglBitmap *
+CoglHandle
 cogl_bitmap_new_from_file (const gchar    *filename,
                            GError        **error)
 {
   CoglBitmap   bmp;
-  
+  CoglBitmap  *ret;
+
   g_return_val_if_fail (error == NULL || *error == NULL, COGL_INVALID_HANDLE);
 
   /* Try loading with imaging backend */
@@ -177,13 +201,8 @@ cogl_bitmap_new_from_file (const gchar    *filename,
 	  *error = NULL;
 	}
     }
-  
-  return (CoglBitmap *) g_memdup (&bmp, sizeof (CoglBitmap));
+
+  ret = g_memdup (&bmp, sizeof (CoglBitmap));
+  return _cogl_bitmap_handle_new (ret);
 }
 
-void
-cogl_bitmap_free (CoglBitmap *bmp)
-{
-  g_free (bmp->data);
-  g_free (bmp);
-}
