@@ -1,13 +1,9 @@
 /*
- * Cogl.
+ * Cogl
  *
- * An OpenGL/GLES Abstraction/Utility Layer
+ * An object oriented GL/GLES Abstraction/Utility Layer
  *
- * Vertex Buffer API: Handle extensible arrays of vertex attributes
- *
- * Copyright (C) 2008  Intel Corporation.
- *
- * Authored by: Robert Bragg <robert@linux.intel.com>
+ * Copyright (C) 2008,2009 Intel Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,7 +16,12 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ *
+ * Authors:
+ *   Robert Bragg <robert@linux.intel.com>
  */
 
 /* XXX: For an overview of the functionality implemented here, please
@@ -126,7 +127,7 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <glib/gprintf.h>
+#include <glib.h>
 
 #include "cogl.h"
 #include "cogl-internal.h"
@@ -135,6 +136,8 @@
 #include "cogl-handle.h"
 #include "cogl-vertex-buffer-private.h"
 #include "cogl-texture-private.h"
+#include "cogl-material-private.h"
+#include "cogl-primitives.h"
 
 #define PAD_FOR_ALIGNMENT(VAR, TYPE_SIZE) \
   (VAR = TYPE_SIZE + ((VAR - 1) & ~(TYPE_SIZE - 1)))
@@ -199,19 +202,16 @@
 #endif /* HAVE_COGL_GL */
 
 static void _cogl_vertex_buffer_free (CoglVertexBuffer *buffer);
+static void _cogl_vertex_buffer_indices_free (CoglVertexBufferIndices *buffer_indices);
 
-COGL_HANDLE_DEFINE (VertexBuffer,
-                    vertex_buffer,
-                    vertex_buffer_handles);
+COGL_HANDLE_DEFINE (VertexBuffer, vertex_buffer);
+COGL_HANDLE_DEFINE (VertexBufferIndices, vertex_buffer_indices);
 
 CoglHandle
 cogl_vertex_buffer_new (guint n_vertices)
 {
   CoglVertexBuffer *buffer = g_slice_alloc (sizeof (CoglVertexBuffer));
 
-  buffer->ref_count = 1;
-  COGL_HANDLE_DEBUG_NEW (CoglVertexBuffer, buffer);
-
   buffer->n_vertices = n_vertices;
 
   buffer->submitted_vbos = NULL;
@@ -415,13 +415,13 @@ get_gl_type_size (CoglVertexBufferAttribFlags flags)
 }
 
 void
-cogl_vertex_buffer_add (CoglHandle handle,
-		        const char *attribute_name,
-			guint8 n_components,
-			GLenum gl_type,
-			gboolean normalized,
-			guint16 stride,
-			const void *pointer)
+cogl_vertex_buffer_add (CoglHandle         handle,
+		        const char        *attribute_name,
+			guint8             n_components,
+			CoglAttributeType  type,
+			gboolean           normalized,
+			guint16            stride,
+			const void        *pointer)
 {
   CoglVertexBuffer *buffer;
   GQuark name_quark = g_quark_from_string (attribute_name);
@@ -489,7 +489,7 @@ cogl_vertex_buffer_add (CoglHandle handle,
   attribute->u.pointer = pointer;
   attribute->texture_unit = texture_unit;
 
-  flags |= get_attribute_gl_type_flag_from_gl_type (gl_type);
+  flags |= get_attribute_gl_type_flag_from_gl_type (type);
   flags |= COGL_VERTEX_BUFFER_ATTRIB_FLAG_ENABLED;
 
   /* Note: We currently just assume, if an attribute is *ever* updated
@@ -747,7 +747,7 @@ filter_strided_attribute (CoglVertexBufferAttrib *attribute,
       CoglVertexBufferVBO *cogl_vbo = tmp->data;
       GList *tmp2;
 
-      if (!cogl_vbo->flags & COGL_VERTEX_BUFFER_VBO_FLAG_STRIDED)
+      if (!(cogl_vbo->flags & COGL_VERTEX_BUFFER_VBO_FLAG_STRIDED))
 	continue;
 
       for (tmp2 = cogl_vbo->attributes; tmp2 != NULL; tmp2 = tmp2->next)
@@ -898,7 +898,12 @@ cogl_vertex_buffer_vbo_free (CoglVertexBufferVBO *cogl_vbo,
 
   if (delete_gl_vbo && cogl_vbo->flags &
       COGL_VERTEX_BUFFER_VBO_FLAG_SUBMITTED)
-    GE (glDeleteBuffers (1, &cogl_vbo->vbo_name));
+    {
+      if (cogl_get_features () & COGL_FEATURE_VBOS)
+	GE (glDeleteBuffers (1, (GLuint *)&cogl_vbo->vbo_name));
+      else
+	g_free (cogl_vbo->vbo_name);
+    }
 
   g_slice_free (CoglVertexBufferVBO, cogl_vbo);
 }
@@ -942,11 +947,19 @@ upload_multipack_vbo_via_map_buffer (CoglVertexBufferVBO *cogl_vbo)
   GList *tmp;
   guint offset = 0;
   char *buf;
+  gboolean fallback =
+    (cogl_get_features () & COGL_FEATURE_VBOS) ? FALSE : TRUE;
 
   _COGL_GET_CONTEXT (ctx, FALSE);
 
-  buf = glMapBuffer (GL_ARRAY_BUFFER, GL_WRITE_ONLY);
-  glGetError();
+  if (!fallback)
+    {
+      buf = glMapBuffer (GL_ARRAY_BUFFER, GL_WRITE_ONLY);
+      glGetError();
+    }
+  else
+    buf = cogl_vbo->vbo_name;
+
   if (!buf)
     return FALSE;
 
@@ -964,7 +977,9 @@ upload_multipack_vbo_via_map_buffer (CoglVertexBufferVBO *cogl_vbo)
       attribute->flags |= COGL_VERTEX_BUFFER_ATTRIB_FLAG_SUBMITTED;
       offset += attribute_size;
     }
-  glUnmapBuffer (GL_ARRAY_BUFFER);
+
+  if (!fallback)
+    glUnmapBuffer (GL_ARRAY_BUFFER);
 
   return TRUE;
 #else
@@ -977,6 +992,8 @@ upload_multipack_vbo_via_buffer_sub_data (CoglVertexBufferVBO *cogl_vbo)
 {
   GList *tmp;
   guint offset = 0;
+  gboolean fallback =
+    (cogl_get_features () & COGL_FEATURE_VBOS) ? FALSE : TRUE;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -988,10 +1005,19 @@ upload_multipack_vbo_via_buffer_sub_data (CoglVertexBufferVBO *cogl_vbo)
 
       PAD_FOR_ALIGNMENT (offset, gl_type_size);
 
-      GE (glBufferSubData (GL_ARRAY_BUFFER,
-			   offset,
-			   attribute_size,
-			   attribute->u.pointer));
+      if (!fallback)
+	{
+	  GE (glBufferSubData (GL_ARRAY_BUFFER,
+			       offset,
+			       attribute_size,
+			       attribute->u.pointer));
+	}
+      else
+	{
+	  char *dest = (char *)cogl_vbo->vbo_name + offset;
+	  memcpy (dest, attribute->u.pointer, attribute_size);
+	}
+
       attribute->u.vbo_offset = offset;
       attribute->flags |= COGL_VERTEX_BUFFER_ATTRIB_FLAG_SUBMITTED;
       offset += attribute_size;
@@ -1002,36 +1028,58 @@ static void
 upload_gl_vbo (CoglVertexBufferVBO *cogl_vbo)
 {
   GLenum usage;
+  gboolean fallback =
+    (cogl_get_features () & COGL_FEATURE_VBOS) ? FALSE : TRUE;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  g_return_if_fail (cogl_vbo->vbo_name != 0);
-
   if (cogl_vbo->flags & COGL_VERTEX_BUFFER_VBO_FLAG_FREQUENT_RESUBMIT)
     usage = GL_DYNAMIC_DRAW;
   else
     usage = GL_STATIC_DRAW;
 
-  GE (glBindBuffer (GL_ARRAY_BUFFER, cogl_vbo->vbo_name));
+  if (!fallback)
+    {
+      g_return_if_fail (cogl_vbo->vbo_name != 0);
+
+      GE (glBindBuffer (GL_ARRAY_BUFFER,
+			GPOINTER_TO_UINT (cogl_vbo->vbo_name)));
+    }
+  else if (cogl_vbo->vbo_name == NULL)
+    {
+      /* If the driver doesn't support VBOs then we simply allocate
+       * a client side fake vbo buffer. Unlike VBOs we can't allocate
+       * without specifying a size which is why we defer allocation
+       * until here. */
+      cogl_vbo->vbo_name = g_malloc (cogl_vbo->vbo_bytes);
+    }
 
   if (cogl_vbo->flags & COGL_VERTEX_BUFFER_VBO_FLAG_STRIDED)
     {
       const void *pointer =
 	prep_strided_vbo_for_upload (cogl_vbo);
-      GE (glBufferData (GL_ARRAY_BUFFER,
-			cogl_vbo->vbo_bytes,
-			pointer,
-			usage));
+      if (!fallback)
+	{
+	  GE (glBufferData (GL_ARRAY_BUFFER,
+			    cogl_vbo->vbo_bytes,
+			    pointer,
+			    usage));
+	}
+      else
+	memcpy (cogl_vbo->vbo_name, pointer, cogl_vbo->vbo_bytes);
     }
   else if (cogl_vbo->flags & COGL_VERTEX_BUFFER_VBO_FLAG_MULTIPACK)
     {
       /* First we make it obvious to the driver that we want to update the
        * whole buffer (without this, the driver is more likley to block
        * if the GPU is busy using the buffer) */
-      GE (glBufferData (GL_ARRAY_BUFFER,
-			cogl_vbo->vbo_bytes,
-			NULL,
-			usage));
+      if (!fallback)
+	{
+	  GE (glBufferData (GL_ARRAY_BUFFER,
+			    cogl_vbo->vbo_bytes,
+			    NULL,
+			    usage));
+	}
 
       /* I think it might depend on the specific driver/HW whether its better
        * to use glMapBuffer here or glBufferSubData here. There is even a good
@@ -1046,10 +1094,16 @@ upload_gl_vbo (CoglVertexBufferVBO *cogl_vbo)
   else
     {
       CoglVertexBufferAttrib *attribute = cogl_vbo->attributes->data;
-      GE (glBufferData (GL_ARRAY_BUFFER,
-			cogl_vbo->vbo_bytes,
-			attribute->u.pointer,
-			usage));
+      if (!fallback)
+	{
+	  GE (glBufferData (GL_ARRAY_BUFFER,
+			    cogl_vbo->vbo_bytes,
+			    attribute->u.pointer,
+			    usage));
+	}
+      else
+	memcpy (cogl_vbo->vbo_name, attribute->u.pointer, cogl_vbo->vbo_bytes);
+
       /* We forget this pointer now since the client will be free
        * to re-use this memory */
       attribute->u.pointer = NULL;
@@ -1058,7 +1112,8 @@ upload_gl_vbo (CoglVertexBufferVBO *cogl_vbo)
 
   cogl_vbo->flags |= COGL_VERTEX_BUFFER_VBO_FLAG_SUBMITTED;
 
-  GE (glBindBuffer (GL_ARRAY_BUFFER, 0));
+  if (!fallback)
+    GE (glBindBuffer (GL_ARRAY_BUFFER, 0));
 }
 
 /* Note: although there ends up being quite a few inner loops involved with
@@ -1117,7 +1172,11 @@ cogl_vertex_buffer_vbo_resolve (CoglVertexBuffer *buffer,
 
   if (!found_target_vbo)
     {
-      GE (glGenBuffers (1, &new_cogl_vbo->vbo_name));
+      if (cogl_get_features () & COGL_FEATURE_VBOS)
+	GE (glGenBuffers (1, (GLuint *)&new_cogl_vbo->vbo_name));
+      else
+	new_cogl_vbo->vbo_name = NULL;
+        /* this will be allocated at upload time */
 
       upload_gl_vbo (new_cogl_vbo);
       *final_vbos = g_list_prepend (*final_vbos, new_cogl_vbo);
@@ -1380,6 +1439,7 @@ cogl_vertex_buffer_submit_real (CoglVertexBuffer *buffer)
   for (tmp = buffer->submitted_vbos; tmp != NULL; tmp = tmp->next)
     cogl_vertex_buffer_vbo_free (tmp->data, TRUE);
   g_list_free (buffer->submitted_vbos);
+  g_list_free (new_vbos);
 
   buffer->submitted_vbos = final_vbos;
 }
@@ -1435,13 +1495,16 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
 {
   GList       *tmp;
   GLenum       gl_type;
+#ifdef MAY_HAVE_PROGRAMABLE_GL
   GLuint       generic_index = 0;
+#endif
   gulong       enable_flags = 0;
   guint        max_texcoord_attrib_unit = 0;
   const GList *layers;
-  guint32      fallback_mask = 0;
-  guint32      disable_mask = ~0;
+  guint32      fallback_layers = 0;
+  guint32      disable_layers = ~0;
   int          i;
+  CoglMaterialFlushOptions options;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -1452,8 +1515,25 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
     {
       CoglVertexBufferVBO *cogl_vbo = tmp->data;
       GList *tmp2;
+      char *base;
+      const GLvoid *pointer;
 
-      GE (glBindBuffer (GL_ARRAY_BUFFER, cogl_vbo->vbo_name));
+      if (cogl_get_features () & COGL_FEATURE_VBOS)
+	{
+	  GE (glBindBuffer (GL_ARRAY_BUFFER,
+			    GPOINTER_TO_UINT (cogl_vbo->vbo_name)));
+	  base = 0;
+	}
+      else
+	base = cogl_vbo->vbo_name;
+
+      /* When GL VBOs are bing used then the "pointer" we pass to
+       * glColorPointer glVertexAttribPointer etc is actually an offset into
+       * the currently bound VBO.
+       *
+       * If we don't have VBO support though, then we must point into
+       * our fake client side VBO.
+       */
 
       for (tmp2 = cogl_vbo->attributes; tmp2 != NULL; tmp2 = tmp2->next)
 	{
@@ -1470,37 +1550,41 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
 	    case COGL_VERTEX_BUFFER_ATTRIB_FLAG_COLOR_ARRAY:
 	      enable_flags |= COGL_ENABLE_COLOR_ARRAY | COGL_ENABLE_BLEND;
 	      /* GE (glEnableClientState (GL_COLOR_ARRAY)); */
+	      pointer = (const GLvoid *)(base + attribute->u.vbo_offset);
 	      GE (glColorPointer (attribute->n_components,
 				  gl_type,
 				  attribute->stride,
-				  (const GLvoid *)attribute->u.vbo_offset));
+				  pointer));
 	      break;
 	    case COGL_VERTEX_BUFFER_ATTRIB_FLAG_NORMAL_ARRAY:
 	      /* FIXME: go through cogl cache to enable normal array */
 	      GE (glEnableClientState (GL_NORMAL_ARRAY));
+	      pointer = (const GLvoid *)(base + attribute->u.vbo_offset);
 	      GE (glNormalPointer (gl_type,
 				   attribute->stride,
-				   (const GLvoid *)attribute->u.vbo_offset));
+				   pointer));
 	      break;
 	    case COGL_VERTEX_BUFFER_ATTRIB_FLAG_TEXTURE_COORD_ARRAY:
               GE (glClientActiveTexture (GL_TEXTURE0 +
                                          attribute->texture_unit));
               GE (glEnableClientState (GL_TEXTURE_COORD_ARRAY));
+	      pointer = (const GLvoid *)(base + attribute->u.vbo_offset);
 	      GE (glTexCoordPointer (attribute->n_components,
 				     gl_type,
 				     attribute->stride,
-				     (const GLvoid *)attribute->u.vbo_offset));
+				     pointer));
               if (attribute->texture_unit > max_texcoord_attrib_unit)
                 max_texcoord_attrib_unit = attribute->texture_unit;
-              disable_mask &= ~(1 << attribute->texture_unit);
+              disable_layers &= ~(1 << attribute->texture_unit);
 	      break;
 	    case COGL_VERTEX_BUFFER_ATTRIB_FLAG_VERTEX_ARRAY:
 	      enable_flags |= COGL_ENABLE_VERTEX_ARRAY;
 	      /* GE (glEnableClientState (GL_VERTEX_ARRAY)); */
+	      pointer = (const GLvoid *)(base + attribute->u.vbo_offset);
 	      GE (glVertexPointer (attribute->n_components,
 				   gl_type,
 				   attribute->stride,
-				   (const GLvoid *)attribute->u.vbo_offset));
+				   pointer));
 	      break;
 	    case COGL_VERTEX_BUFFER_ATTRIB_FLAG_CUSTOM_ARRAY:
 	      {
@@ -1511,13 +1595,13 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
 		  normalized = GL_TRUE;
 		/* FIXME: go through cogl cache to enable generic array */
 		GE (glEnableVertexAttribArray (generic_index++));
+		pointer = (const GLvoid *)(base + attribute->u.vbo_offset);
 		GE (glVertexAttribPointer (generic_index,
 					   attribute->n_components,
 					   gl_type,
 					   normalized,
 					   attribute->stride,
-					   (const GLvoid *)
-					    attribute->u.vbo_offset));
+					   pointer));
 #endif
 	      }
 	      break;
@@ -1534,8 +1618,14 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
     {
       CoglHandle layer = (CoglHandle)tmp->data;
       CoglHandle tex_handle = cogl_material_layer_get_texture (layer);
-      CoglTexture *texture =
-        _cogl_texture_pointer_from_handle (tex_handle);
+      CoglTexture *texture;
+
+      /* invalid textures will be handled correctly in
+       * _cogl_material_flush_layers_gl_state */
+      if (tex_handle == COGL_INVALID_HANDLE)
+        continue;
+
+      texture = _cogl_texture_pointer_from_handle (tex_handle);
 
       if (cogl_texture_is_sliced (tex_handle)
           || _cogl_texture_span_has_waste (texture, 0, 0))
@@ -1556,18 +1646,28 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
            * vertices once for each layer, each time with a fiddled texture
            * matrix.
            */
-          fallback_mask |= (1 << i);
+          fallback_layers |= (1 << i);
         }
     }
 
-  cogl_material_flush_gl_state (ctx->source_material,
-                                COGL_MATERIAL_FLUSH_FALLBACK_MASK,
-                                fallback_mask,
-                                COGL_MATERIAL_FLUSH_DISABLE_MASK,
-                                disable_mask,
-                                NULL);
+  for (i = max_texcoord_attrib_unit + 1; i < ctx->n_texcoord_arrays_enabled; i++)
+    {
+      GE (glClientActiveTexture (GL_TEXTURE0 + i));
+      GE (glDisableClientState (GL_TEXTURE_COORD_ARRAY));
+    }
+  ctx->n_texcoord_arrays_enabled = max_texcoord_attrib_unit + 1;
 
-  enable_flags |= cogl_material_get_cogl_enable_flags (ctx->source_material);
+  options.flags =
+    COGL_MATERIAL_FLUSH_FALLBACK_MASK |
+    COGL_MATERIAL_FLUSH_DISABLE_MASK;
+  options.fallback_layers = fallback_layers;
+  options.disable_layers = disable_layers;
+
+  _cogl_material_flush_gl_state (ctx->source_material, &options);
+  enable_flags |= _cogl_material_get_cogl_enable_flags (ctx->source_material);
+
+  if (ctx->enable_backface_culling)
+    enable_flags |= COGL_ENABLE_BACKFACE_CULLING;
 
   cogl_enable (enable_flags);
 }
@@ -1577,16 +1677,18 @@ disable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
 {
   GList *tmp;
   GLenum gl_type;
+#ifdef MAY_HAVE_PROGRAMABLE_GL
   GLuint generic_index = 0;
+#endif
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
   /* Disable all the client state that cogl doesn't currently know
    * about:
    */
-  GE (glBindBuffer (GL_ARRAY_BUFFER, 0));
+  if (cogl_get_features () & COGL_FEATURE_VBOS)
+    GE (glBindBuffer (GL_ARRAY_BUFFER, 0));
 
-  generic_index = 0;
   for (tmp = buffer->submitted_vbos; tmp != NULL; tmp = tmp->next)
     {
       CoglVertexBufferVBO *cogl_vbo = tmp->data;
@@ -1633,21 +1735,24 @@ disable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
 }
 
 void
-cogl_vertex_buffer_draw (CoglHandle handle,
-		         GLenum mode,
-		         GLint first,
-		         GLsizei count)
+cogl_vertex_buffer_draw (CoglHandle       handle,
+		         CoglVerticesMode mode,
+		         int              first,
+		         int              count)
 {
   CoglVertexBuffer *buffer;
 
   if (!cogl_is_vertex_buffer (handle))
     return;
 
+  _cogl_journal_flush ();
+  cogl_clip_ensure ();
+
   buffer = _cogl_vertex_buffer_pointer_from_handle (handle);
 
-  enable_state_for_drawing_buffer (buffer);
-
+  cogl_clip_ensure ();
   _cogl_current_matrix_state_flush ();
+  enable_state_for_drawing_buffer (buffer);
 
   /* FIXME: flush cogl cache */
   GE (glDrawArrays (mode, first, count));
@@ -1655,33 +1760,151 @@ cogl_vertex_buffer_draw (CoglHandle handle,
   disable_state_for_drawing_buffer (buffer);
 }
 
+static int
+get_indices_type_size (GLuint indices_type)
+{
+  if (indices_type == GL_UNSIGNED_BYTE)
+    return sizeof (GLubyte);
+  if (indices_type == GL_UNSIGNED_SHORT)
+    return sizeof (GLushort);
+  else
+    {
+      g_critical ("Unknown indices type %d\n", indices_type);
+      return 0;
+    }
+}
+
+CoglHandle
+cogl_vertex_buffer_indices_new (CoglIndicesType  indices_type,
+                                const void      *indices_array,
+                                int              indices_len)
+{
+  gboolean fallback =
+    (cogl_get_features () & COGL_FEATURE_VBOS) ? FALSE : TRUE;
+  size_t indices_bytes;
+  CoglVertexBufferIndices *indices;
+
+  _COGL_GET_CONTEXT (ctx, 0);
+
+  indices = g_slice_alloc (sizeof (CoglVertexBufferIndices));
+
+  if (indices_type == COGL_INDICES_TYPE_UNSIGNED_BYTE)
+    indices->type = GL_UNSIGNED_BYTE;
+  else if (indices_type == COGL_INDICES_TYPE_UNSIGNED_SHORT)
+    indices->type = GL_UNSIGNED_SHORT;
+  else
+    {
+      g_critical ("unknown indices type %d", indices_type);
+      g_slice_free (CoglVertexBufferIndices, indices);
+      return 0;
+    }
+
+  indices_bytes = get_indices_type_size (indices->type) * indices_len;
+  if (fallback)
+    {
+      indices->vbo_name = g_malloc (indices_len);
+      memcpy (indices->vbo_name, indices_array, indices_bytes);
+    }
+  else
+    {
+      GE (glGenBuffers (1, (GLuint *)&indices->vbo_name));
+      GE (glBindBuffer (GL_ELEMENT_ARRAY_BUFFER,
+			GPOINTER_TO_UINT (indices->vbo_name)));
+      GE (glBufferData (GL_ELEMENT_ARRAY_BUFFER,
+                        indices_bytes,
+                        indices_array,
+                        GL_STATIC_DRAW));
+      GE (glBindBuffer (GL_ELEMENT_ARRAY_BUFFER, 0));
+    }
+
+  return _cogl_vertex_buffer_indices_handle_new (indices);
+}
+
+CoglIndicesType
+cogl_vertex_buffer_indices_get_type (CoglHandle indices_handle)
+{
+  CoglVertexBufferIndices *indices = NULL;
+
+  if (!cogl_is_vertex_buffer_indices (indices_handle))
+    return COGL_INDICES_TYPE_UNSIGNED_SHORT;
+
+  indices = _cogl_vertex_buffer_indices_pointer_from_handle (indices_handle);
+
+  if (indices->type == GL_UNSIGNED_BYTE)
+    return COGL_INDICES_TYPE_UNSIGNED_BYTE;
+  else if (indices->type == GL_UNSIGNED_SHORT)
+    return COGL_INDICES_TYPE_UNSIGNED_SHORT;
+  else
+    {
+      g_critical ("unknown indices type %d", indices->type);
+      return COGL_INDICES_TYPE_UNSIGNED_SHORT;
+    }
+}
+
 void
-cogl_vertex_buffer_draw_elements (CoglHandle handle,
-			          GLenum mode,
-			          GLuint min_index,
-			          GLuint max_index,
-			          GLsizei count,
-			          GLenum indices_type,
-			          const GLvoid *indices)
+_cogl_vertex_buffer_indices_free (CoglVertexBufferIndices *indices)
+{
+  gboolean fallback =
+    (cogl_get_features () & COGL_FEATURE_VBOS) ? FALSE : TRUE;
+
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  if (fallback)
+    g_free (indices->vbo_name);
+  else
+    GE (glDeleteBuffers (1, (GLuint *)&indices->vbo_name));
+
+  g_slice_free (CoglVertexBufferIndices, indices);
+}
+
+void
+cogl_vertex_buffer_draw_elements (CoglHandle       handle,
+			          CoglVerticesMode mode,
+                                  CoglHandle       indices_handle,
+                                  int              min_index,
+                                  int              max_index,
+                                  int              indices_offset,
+                                  int              count)
 {
   CoglVertexBuffer *buffer;
+  gboolean fallback =
+    (cogl_get_features () & COGL_FEATURE_VBOS) ? FALSE : TRUE;
+  size_t byte_offset;
+  CoglVertexBufferIndices *indices = NULL;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
   if (!cogl_is_vertex_buffer (handle))
     return;
 
+  _cogl_journal_flush ();
+  cogl_clip_ensure ();
+
   buffer = _cogl_vertex_buffer_pointer_from_handle (handle);
 
+  if (!cogl_is_vertex_buffer_indices (indices_handle))
+    return;
+
+  indices = _cogl_vertex_buffer_indices_pointer_from_handle (indices_handle);
+
+  cogl_clip_ensure ();
+  _cogl_current_matrix_state_flush ();
   enable_state_for_drawing_buffer (buffer);
 
-  _cogl_current_matrix_state_flush ();
+  byte_offset = indices_offset * get_indices_type_size (indices->type);
+  if (fallback)
+    byte_offset = (size_t)(((char *)indices->vbo_name) + byte_offset);
+  else
+    GE (glBindBuffer (GL_ELEMENT_ARRAY_BUFFER,
+                      GPOINTER_TO_UINT (indices->vbo_name)));
 
   /* FIXME: flush cogl cache */
   GE (glDrawRangeElements (mode, min_index, max_index,
-                           count, indices_type, indices));
+                           count, indices->type, (void *)byte_offset));
 
   disable_state_for_drawing_buffer (buffer);
+
+  GE (glBindBuffer (GL_ELEMENT_ARRAY_BUFFER, 0));
 }
 
 static void
@@ -1691,9 +1914,92 @@ _cogl_vertex_buffer_free (CoglVertexBuffer *buffer)
 
   for (tmp = buffer->submitted_vbos; tmp != NULL; tmp = tmp->next)
     cogl_vertex_buffer_vbo_free (tmp->data, TRUE);
+  g_list_free (buffer->submitted_vbos);
+
   for (tmp = buffer->new_attributes; tmp != NULL; tmp = tmp->next)
     cogl_vertex_buffer_attribute_free (tmp->data);
+  g_list_free (buffer->new_attributes);
 
   g_slice_free (CoglVertexBuffer, buffer);
 }
 
+CoglHandle
+cogl_vertex_buffer_indices_get_for_quads (guint n_indices)
+{
+  _COGL_GET_CONTEXT (ctx, COGL_INVALID_HANDLE);
+
+  /* Check if the indices would fit in a byte array */
+  if (n_indices <= 256 / 4 * 6)
+    {
+      /* Generate the byte array if we haven't already */
+      if (ctx->quad_indices_byte == COGL_INVALID_HANDLE)
+        {
+          guint8 *byte_array = g_malloc (256 / 4 * 6 * sizeof (guint8));
+          guint8 *p = byte_array;
+          int i, vert_num = 0;
+
+          for (i = 0; i < 256 / 4; i++)
+            {
+              *(p++) = vert_num + 0;
+              *(p++) = vert_num + 1;
+              *(p++) = vert_num + 2;
+              *(p++) = vert_num + 0;
+              *(p++) = vert_num + 2;
+              *(p++) = vert_num + 3;
+              vert_num += 4;
+            }
+
+          ctx->quad_indices_byte
+            = cogl_vertex_buffer_indices_new (COGL_INDICES_TYPE_UNSIGNED_BYTE,
+                                              byte_array,
+                                              256 / 4 * 6);
+
+          g_free (byte_array);
+        }
+
+      return ctx->quad_indices_byte;
+    }
+  else
+    {
+      if (ctx->quad_indices_short_len < n_indices)
+        {
+          guint16 *short_array;
+          guint16 *p;
+          int i, vert_num = 0;
+
+          if (ctx->quad_indices_short != COGL_INVALID_HANDLE)
+            cogl_handle_unref (ctx->quad_indices_short);
+          /* Pick a power of two >= MAX (512, n_indices) */
+          if (ctx->quad_indices_short_len == 0)
+            ctx->quad_indices_short_len = 512;
+          while (ctx->quad_indices_short_len < n_indices)
+            ctx->quad_indices_short_len *= 2;
+
+          /* Over-allocate to generate a whole number of quads */
+          p = short_array = g_malloc ((ctx->quad_indices_short_len
+                                       + 5) / 6 * 6
+                                      * sizeof (guint16));
+
+          /* Fill in the complete quads */
+          for (i = 0; i < ctx->quad_indices_short_len; i += 6)
+            {
+              *(p++) = vert_num + 0;
+              *(p++) = vert_num + 1;
+              *(p++) = vert_num + 2;
+              *(p++) = vert_num + 0;
+              *(p++) = vert_num + 2;
+              *(p++) = vert_num + 3;
+              vert_num += 4;
+            }
+
+          ctx->quad_indices_short
+            = cogl_vertex_buffer_indices_new (COGL_INDICES_TYPE_UNSIGNED_SHORT,
+                                              short_array,
+                                              ctx->quad_indices_short_len);
+
+          g_free (short_array);
+        }
+
+      return ctx->quad_indices_short;
+    }
+}
