@@ -1,11 +1,9 @@
 /*
- * Clutter COGL
+ * Cogl
  *
- * A basic GL/GLES Abstraction/Utility Layer
+ * An object oriented GL/GLES Abstraction/Utility Layer
  *
- * Authored By Matthew Allum  <mallum@openedhand.com>
- *
- * Copyright (C) 2008 OpenedHand
+ * Copyright (C) 2008,2009 Intel Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -42,7 +40,7 @@
 
 static void _cogl_program_free (CoglProgram *program);
 
-COGL_HANDLE_DEFINE (Program, program, program_handles);
+COGL_HANDLE_DEFINE (Program, program);
 
 static void
 _cogl_program_free (CoglProgram *program)
@@ -52,7 +50,7 @@ _cogl_program_free (CoglProgram *program)
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
   /* Unref all of the attached shaders */
-  g_slist_foreach (program->attached_shaders, (GFunc) cogl_shader_unref, NULL);
+  g_slist_foreach (program->attached_shaders, (GFunc) cogl_handle_unref, NULL);
   /* Destroy the list */
   g_slist_free (program->attached_shaders);
 
@@ -75,13 +73,10 @@ cogl_create_program (void)
   CoglProgram *program;
 
   program = g_slice_new (CoglProgram);
-  program->ref_count = 1;
   program->attached_shaders = NULL;
   memset (program->custom_uniform_names, 0,
 	  COGL_GLES2_NUM_CUSTOM_UNIFORMS * sizeof (char *));
 
-  COGL_HANDLE_DEBUG_NEW (program, program);
-
   return _cogl_program_handle_new (program);
 }
 
@@ -92,14 +87,14 @@ cogl_program_attach_shader (CoglHandle program_handle,
   CoglProgram *program;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-  
+
   if (!cogl_is_program (program_handle) || !cogl_is_shader (shader_handle))
     return;
 
   program = _cogl_program_pointer_from_handle (program_handle);
   program->attached_shaders
     = g_slist_prepend (program->attached_shaders,
-		       cogl_shader_ref (shader_handle));
+		       cogl_handle_ref (shader_handle));
 
   /* Whenever the shader changes we will need to relink the program
      with the fixed functionality shaders so we should forget the
@@ -119,7 +114,7 @@ void
 cogl_program_use (CoglHandle handle)
 {
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-  
+
   if (handle != COGL_INVALID_HANDLE && !cogl_is_program (handle))
     return;
 
@@ -127,7 +122,7 @@ cogl_program_use (CoglHandle handle)
   ctx->gles2.settings_dirty = TRUE;
 }
 
-COGLint
+int
 cogl_program_get_uniform_location (CoglHandle   handle,
                                    const gchar *uniform_name)
 {
@@ -160,21 +155,21 @@ cogl_program_get_uniform_location (CoglHandle   handle,
 }
 
 void
-cogl_program_uniform_1f (COGLint uniform_no,
+cogl_program_uniform_1f (int uniform_no,
                          gfloat  value)
 {
   cogl_program_uniform_float (uniform_no, 1, 1, &value);
 }
 
 void
-cogl_program_uniform_1i (COGLint uniform_no,
+cogl_program_uniform_1i (int uniform_no,
                          gint    value)
 {
   cogl_program_uniform_int (uniform_no, 1, 1, &value);
 }
 
 static void
-cogl_program_uniform_x (COGLint uniform_no,
+cogl_program_uniform_x (int uniform_no,
 			gint size,
 			gint count,
 			CoglBoxedType type,
@@ -220,7 +215,7 @@ cogl_program_uniform_x (COGLint uniform_no,
 }
 
 void
-cogl_program_uniform_float (COGLint  uniform_no,
+cogl_program_uniform_float (int  uniform_no,
                             gint     size,
                             gint     count,
                             const GLfloat *value)
@@ -230,7 +225,7 @@ cogl_program_uniform_float (COGLint  uniform_no,
 }
 
 void
-cogl_program_uniform_int (COGLint  uniform_no,
+cogl_program_uniform_int (int  uniform_no,
 			  gint   size,
 			  gint   count,
 			  const GLint *value)
@@ -240,7 +235,7 @@ cogl_program_uniform_int (COGLint  uniform_no,
 }
 
 void
-cogl_program_uniform_matrix (COGLint   uniform_no,
+cogl_program_uniform_matrix (int   uniform_no,
                              gint      size,
                              gint      count,
                              gboolean  transpose,
@@ -301,7 +296,7 @@ cogl_program_use (CoglHandle program_handle)
 {
 }
 
-COGLint
+int
 cogl_program_get_uniform_location (CoglHandle   program_handle,
                                    const gchar *uniform_name)
 {
@@ -309,13 +304,13 @@ cogl_program_get_uniform_location (CoglHandle   program_handle,
 }
 
 void
-cogl_program_uniform_1f (COGLint uniform_no,
+cogl_program_uniform_1f (int uniform_no,
                          gfloat  value)
 {
 }
 
 void
-cogl_program_uniform_float (COGLint  uniform_no,
+cogl_program_uniform_float (int  uniform_no,
                             gint     size,
                             gint     count,
                             const GLfloat *value)
@@ -323,15 +318,15 @@ cogl_program_uniform_float (COGLint  uniform_no,
 }
 
 void
-cogl_program_uniform_int (COGLint  uniform_no,
+cogl_program_uniform_int (int  uniform_no,
                           gint     size,
                           gint     count,
-                          const COGLint *value)
+                          const int *value)
 {
 }
 
 void
-cogl_program_uniform_matrix (COGLint   uniform_no,
+cogl_program_uniform_matrix (int   uniform_no,
                              gint      size,
                              gint      count,
                              gboolean  transpose,
