@@ -128,6 +128,7 @@ clutter_stage_osx_state_update (ClutterStageOSX   *self,
 - (void) drawRect: (NSRect) bounds
 {
   clutter_actor_paint (CLUTTER_ACTOR (self->stage_osx->wrapper));
+  cogl_flush ();
   [[self openGLContext] flushBuffer];
 }
 
@@ -295,11 +296,7 @@ clutter_stage_osx_realize (ClutterActor *actor)
 
   CLUTTER_OSX_POOL_RELEASE();
 
-  /* FIXME: ClutterStage:realize is using the class pointer directly rather
-   * than clutter_actor_realize which would set the REALIZED flag for us.
-   */
-  CLUTTER_NOTE (BACKEND, "Marking stage as realized");
-  CLUTTER_ACTOR_SET_FLAGS (actor, CLUTTER_ACTOR_REALIZED);
+  CLUTTER_NOTE (BACKEND, "Stage successfully realized");
 }
 
 static void
@@ -321,8 +318,6 @@ clutter_stage_osx_unrealize (ClutterActor *actor)
   self->window = NULL;
 
   CLUTTER_OSX_POOL_RELEASE();
-
-  CLUTTER_ACTOR_UNSET_FLAGS (actor, CLUTTER_ACTOR_REALIZED);
 }
 
 static void
@@ -367,9 +362,9 @@ clutter_stage_osx_hide (ClutterActor *actor)
 
 static void
 clutter_stage_osx_get_preferred_width (ClutterActor *actor,
-                                       ClutterUnit   for_height,
-                                       ClutterUnit  *min_width_p,
-                                       ClutterUnit  *natural_width_p)
+                                       gfloat        for_height,
+                                       gfloat       *min_width_p,
+                                       gfloat       *natural_width_p)
 {
   ClutterStageOSX *self = CLUTTER_STAGE_OSX (actor);
   gboolean is_resizable;
@@ -394,9 +389,9 @@ clutter_stage_osx_get_preferred_width (ClutterActor *actor,
 
 static void
 clutter_stage_osx_get_preferred_height (ClutterActor *actor,
-                                        ClutterUnit   for_width,
-                                        ClutterUnit  *min_height_p,
-                                        ClutterUnit  *natural_height_p)
+                                        gfloat        for_width,
+                                        gfloat       *min_height_p,
+                                        gfloat       *natural_height_p)
 {
   ClutterStageOSX *self = CLUTTER_STAGE_OSX (actor);
   gboolean is_resizable;
@@ -420,9 +415,9 @@ clutter_stage_osx_get_preferred_height (ClutterActor *actor,
 }
 
 static void
-clutter_stage_osx_allocate (ClutterActor          *actor,
-                            const ClutterActorBox *box,
-                            gboolean               origin_changed)
+clutter_stage_osx_allocate (ClutterActor           *actor,
+                            const ClutterActorBox  *box,
+                            ClutterAllocationFlags  flags)
 {
   ClutterStageOSX *self = CLUTTER_STAGE_OSX (actor);
   ClutterActorClass *parent_class;
@@ -452,7 +447,7 @@ clutter_stage_osx_allocate (ClutterActor          *actor,
 
   /* chain up */
   parent_class = CLUTTER_ACTOR_CLASS (clutter_stage_osx_parent_class);
-  parent_class->allocate (actor, box, origin_changed);
+  parent_class->allocate (actor, box, flags);
 }
 
 /*************************************************************************/
