@@ -36,6 +36,7 @@
 
 #include "cogl-pango-private.h"
 #include "cogl-pango-glyph-cache.h"
+#include "cogl-pango-display-list.h"
 
 struct _CoglPangoRenderer
 {
@@ -46,16 +47,11 @@ struct _CoglPangoRenderer
   /* The material used for solid fills. (boxes, rectangles + trapezoids) */
   CoglHandle solid_material;
 
-  /* Two caches of glyphs as textures, one with mipmapped textures and
-     one without */
+  /* Caches of glyphs as textures */
   CoglPangoGlyphCache *glyph_cache;
-  CoglPangoGlyphCache *mipmapped_glyph_cache;
 
-  gboolean use_mipmapping;
-
-  /* Array of rectangles to draw from the current texture */
-  GArray *glyph_rectangles;
-  CoglHandle glyph_texture;
+  /* The current display list that is being built */
+  CoglPangoDisplayList *display_list;
 };
 
 struct _CoglPangoRendererClass
@@ -63,19 +59,18 @@ struct _CoglPangoRendererClass
   PangoRendererClass class_instance;
 };
 
-static void
-cogl_pango_renderer_glyphs_end (CoglPangoRenderer *priv)
+typedef struct _CoglPangoRendererQdata CoglPangoRendererQdata;
+
+/* An instance of this struct gets attached to each PangoLayout to
+   cache the VBO and to detect changes to the layout */
+struct _CoglPangoRendererQdata
 {
-  if (priv->glyph_rectangles->len > 0)
-    {
-      float *rectangles = (float *) priv->glyph_rectangles->data;
-      cogl_material_set_layer (priv->glyph_material, 0, priv->glyph_texture);
-      cogl_set_source (priv->glyph_material);
-      cogl_rectangles_with_texture_coords (rectangles,
-                                           priv->glyph_rectangles->len / 8);
-      g_array_set_size (priv->glyph_rectangles, 0);
-    }
-}
+  /* The cache of the geometry for the layout */
+  CoglPangoDisplayList *display_list;
+  /* A reference to the first line of the layout. This is just used to
+     detect changes */
+  PangoLayoutLine *first_line;
+};
 
 static void
 cogl_pango_renderer_draw_glyph (CoglPangoRenderer        *priv,
@@ -84,25 +79,19 @@ cogl_pango_renderer_draw_glyph (CoglPangoRenderer        *priv,
                                 float                     y1)
 {
   float x2, y2;
-  float *p;
 
-  if (priv->glyph_rectangles->len > 0
-      && priv->glyph_texture != cache_value->texture)
-    cogl_pango_renderer_glyphs_end (priv);
-
-  priv->glyph_texture = cache_value->texture;
+  g_return_if_fail (priv->display_list != NULL);
 
   x2 = x1 + (float) cache_value->draw_width;
   y2 = y1 + (float) cache_value->draw_height;
 
-  g_array_set_size (priv->glyph_rectangles, priv->glyph_rectangles->len + 8);
-  p = &g_array_index (priv->glyph_rectangles, float,
-                      priv->glyph_rectangles->len - 8);
-
-  *(p++) = x1;               *(p++) = y1;
-  *(p++) = x2;               *(p++) = y2;
-  *(p++) = cache_value->tx1; *(p++) = cache_value->ty1;
-  *(p++) = cache_value->tx2; *(p++) = cache_value->ty2;
+  _cogl_pango_display_list_add_texture (priv->display_list,
+                                        cache_value->texture,
+                                        x1, y1, x2, y2,
+                                        cache_value->tx1,
+                                        cache_value->ty1,
+                                        cache_value->tx2,
+                                        cache_value->ty2);
 }
 
 static void cogl_pango_renderer_finalize (GObject *object);
@@ -135,38 +124,27 @@ cogl_pango_renderer_init (CoglPangoRenderer *priv)
 
   /* The default combine mode of materials is to modulate (A x B) the texture
    * RGBA channels with the RGBA channels of the previous layer (which in our
-   * case is just the solid font color)
+   * case is just the font color)
    *
-   * Since our glyph cache textures are component alpha textures, and so the
-   * RGB channels are defined as (0, 0, 0) we don't want to modulate the RGB
-   * channels, instead we want to simply replace them with our solid font
-   * color...
+   * Since the RGB for an alpha texture is defined as 0, this gives us:
    *
-   * XXX: we could really do with a neat string based way for describing
-   * combine modes, like: "REPLACE(PREVIOUS[RGB])"
-   * XXX: potentially materials could have a fuzzy default combine mode
-   * such that they default to this for component alpha textures? This would
-   * give the same semantics as the old-style GL_MODULATE mode but sounds a
-   * bit hacky.
+   *  result.rgb = color.rgb * 0
+   *  result.a = color.a * texture.a
+   *
+   * What we want is premultiplied rgba values:
+   *
+   *  result.rgba = color.rgb * texture.a
+   *  result.a = color.a * texture.a
    */
-  cogl_material_set_layer_combine_function (
-                                    priv->glyph_material,
-                                    0, /* layer */
-                                    COGL_MATERIAL_LAYER_COMBINE_CHANNELS_RGB,
-                                    COGL_MATERIAL_LAYER_COMBINE_FUNC_REPLACE);
-  cogl_material_set_layer_combine_arg_src (
-                                    priv->glyph_material,
-                                    0, /* layer */
-                                    0, /* arg */
-                                    COGL_MATERIAL_LAYER_COMBINE_CHANNELS_RGB,
-                                    COGL_MATERIAL_LAYER_COMBINE_SRC_PREVIOUS);
+  cogl_material_set_layer_combine (priv->glyph_material, 0, /* layer */
+                                   "RGBA = MODULATE (PREVIOUS, TEXTURE[A])",
+                                   NULL);
 
   priv->solid_material = cogl_material_new ();
 
-  priv->glyph_cache = cogl_pango_glyph_cache_new (FALSE);
-  priv->mipmapped_glyph_cache = cogl_pango_glyph_cache_new (TRUE);
-  priv->use_mipmapping = FALSE;
-  priv->glyph_rectangles = g_array_new (FALSE, FALSE, sizeof (float));
+  priv->glyph_cache = cogl_pango_glyph_cache_new ();
+
+  _cogl_pango_renderer_set_use_mipmapping (priv, FALSE);
 }
 
 static void
@@ -187,9 +165,7 @@ cogl_pango_renderer_finalize (GObject *object)
 {
   CoglPangoRenderer *priv = COGL_PANGO_RENDERER (object);
 
-  cogl_pango_glyph_cache_free (priv->mipmapped_glyph_cache);
   cogl_pango_glyph_cache_free (priv->glyph_cache);
-  g_array_free (priv->glyph_rectangles, TRUE);
 
   G_OBJECT_CLASS (cogl_pango_renderer_parent_class)->finalize (object);
 }
@@ -211,6 +187,27 @@ cogl_pango_get_renderer_from_context (PangoContext *context)
   return COGL_PANGO_RENDERER (renderer);
 }
 
+static GQuark
+cogl_pango_render_get_qdata_key (void)
+{
+  static GQuark key = 0;
+
+  if (G_UNLIKELY (key == 0))
+    key = g_quark_from_static_string ("CoglPangoDisplayList");
+
+  return key;
+}
+
+static void
+cogl_pango_render_qdata_destroy (CoglPangoRendererQdata *qdata)
+{
+  if (qdata->display_list)
+    _cogl_pango_display_list_free (qdata->display_list);
+  if (qdata->first_line)
+    pango_layout_line_unref (qdata->first_line);
+  g_slice_free (CoglPangoRendererQdata, qdata);
+}
+
 /**
  * cogl_pango_render_layout_subpixel:
  * @layout: a #PangoLayout
@@ -230,18 +227,67 @@ cogl_pango_render_layout_subpixel (PangoLayout     *layout,
 				   const CoglColor *color,
 				   int              flags)
 {
-  PangoContext      *context;
-  CoglPangoRenderer *priv;
+  PangoContext           *context;
+  CoglPangoRenderer      *priv;
+  CoglPangoRendererQdata *qdata;
 
   context = pango_layout_get_context (layout);
   priv = cogl_pango_get_renderer_from_context (context);
   if (G_UNLIKELY (!priv))
     return;
 
-  cogl_material_set_color (priv->glyph_material, color);
-  cogl_material_set_color (priv->solid_material, color);
+  qdata = g_object_get_qdata (G_OBJECT (layout),
+                              cogl_pango_render_get_qdata_key ());
 
-  pango_renderer_draw_layout (PANGO_RENDERER (priv), layout, x, y);
+  if (qdata == NULL)
+    {
+      qdata = g_slice_new0 (CoglPangoRendererQdata);
+      g_object_set_qdata_full (G_OBJECT (layout),
+                               cogl_pango_render_get_qdata_key (),
+                               qdata,
+                               (GDestroyNotify)
+                               cogl_pango_render_qdata_destroy);
+    }
+
+  /* Check if the layout has changed since the last build of the
+     display list. This trick was suggested by Behdad Esfahbod here:
+     http://mail.gnome.org/archives/gtk-i18n-list/2009-May/msg00019.html */
+  if (qdata->display_list && qdata->first_line
+      && qdata->first_line->layout != layout)
+    {
+      _cogl_pango_display_list_free (qdata->display_list);
+      qdata->display_list = NULL;
+    }
+
+  if (qdata->display_list == NULL)
+    {
+      qdata->display_list = _cogl_pango_display_list_new ();
+
+      priv->display_list = qdata->display_list;
+      pango_renderer_draw_layout (PANGO_RENDERER (priv), layout, 0, 0);
+      priv->display_list = NULL;
+    }
+
+  cogl_push_matrix ();
+  cogl_translate (x / (gfloat) PANGO_SCALE, y / (gfloat) PANGO_SCALE, 0);
+  _cogl_pango_display_list_render (qdata->display_list,
+                                   color,
+                                   priv->glyph_material,
+                                   priv->solid_material);
+  cogl_pop_matrix ();
+
+  /* Keep a reference to the first line of the layout so we can detect
+     changes */
+  if (qdata->first_line)
+    {
+      pango_layout_line_unref (qdata->first_line);
+      qdata->first_line = NULL;
+    }
+  if (pango_layout_get_line_count (layout) > 0)
+    {
+      qdata->first_line = pango_layout_get_line (layout, 0);
+      pango_layout_line_ref (qdata->first_line);
+    }
 }
 
 /**
@@ -295,30 +341,48 @@ cogl_pango_render_layout_line (PangoLayoutLine *line,
   if (G_UNLIKELY (!priv))
     return;
 
-  cogl_material_set_color (priv->glyph_material, color);
-  cogl_material_set_color (priv->solid_material, color);
+  priv->display_list = _cogl_pango_display_list_new ();
 
   pango_renderer_draw_layout_line (PANGO_RENDERER (priv), line, x, y);
+
+  _cogl_pango_display_list_render (priv->display_list,
+                                   color,
+                                   priv->glyph_material,
+                                   priv->solid_material);
+
+  _cogl_pango_display_list_free (priv->display_list);
+  priv->display_list = NULL;
 }
 
 void
 _cogl_pango_renderer_clear_glyph_cache (CoglPangoRenderer *renderer)
 {
   cogl_pango_glyph_cache_clear (renderer->glyph_cache);
-  cogl_pango_glyph_cache_clear (renderer->mipmapped_glyph_cache);
 }
 
 void
 _cogl_pango_renderer_set_use_mipmapping (CoglPangoRenderer *renderer,
-					    gboolean value)
+                                         gboolean value)
 {
-  renderer->use_mipmapping = value;
+  if (value)
+    cogl_material_set_layer_filters (renderer->glyph_material, 0,
+                                     COGL_MATERIAL_FILTER_LINEAR_MIPMAP_LINEAR,
+                                     COGL_MATERIAL_FILTER_LINEAR);
+  else
+    cogl_material_set_layer_filters (renderer->glyph_material, 0,
+                                     COGL_MATERIAL_FILTER_LINEAR,
+                                     COGL_MATERIAL_FILTER_LINEAR);
 }
 
 gboolean
 _cogl_pango_renderer_get_use_mipmapping (CoglPangoRenderer *renderer)
 {
-  return renderer->use_mipmapping;
+  const GList *layers = cogl_material_get_layers (renderer->glyph_material);
+
+  g_return_val_if_fail (layers != NULL, FALSE);
+
+  return (cogl_material_layer_get_min_filter (layers->data)
+          == COGL_MATERIAL_FILTER_LINEAR_MIPMAP_LINEAR);
 }
 
 static CoglPangoGlyphCacheValue *
@@ -328,12 +392,8 @@ cogl_pango_renderer_get_cached_glyph (PangoRenderer *renderer,
 {
   CoglPangoRenderer *priv = COGL_PANGO_RENDERER (renderer);
   CoglPangoGlyphCacheValue *value;
-  CoglPangoGlyphCache *glyph_cache;
 
-  glyph_cache = priv->use_mipmapping ? priv->mipmapped_glyph_cache
-                                     : priv->glyph_cache;
-
-  value = cogl_pango_glyph_cache_lookup (glyph_cache, font, glyph);
+  value = cogl_pango_glyph_cache_lookup (priv->glyph_cache, font, glyph);
   if (value == NULL)
     {
       cairo_surface_t *surface;
@@ -365,7 +425,7 @@ cogl_pango_renderer_get_cached_glyph (PangoRenderer *renderer,
 
       /* Copy the glyph to the cache */
       value =
-        cogl_pango_glyph_cache_set (glyph_cache, font, glyph,
+        cogl_pango_glyph_cache_set (priv->glyph_cache, font, glyph,
                                     cairo_image_surface_get_data (surface),
                                     cairo_image_surface_get_width (surface),
                                     cairo_image_surface_get_height (surface),
@@ -436,19 +496,17 @@ cogl_pango_renderer_set_color_for_part (PangoRenderer   *renderer,
   if (pango_color)
     {
       CoglColor color;
-      guint8 red = pango_color->red >> 8;
-      guint8 green = pango_color->green >> 8;
-      guint8 blue = pango_color->blue >> 8;
-      guint8 alpha;
 
-      cogl_material_get_color (priv->solid_material, &color);
-      alpha = cogl_color_get_alpha_byte (&color);
+      cogl_color_set_from_4ub (&color,
+                               pango_color->red >> 8,
+                               pango_color->green >> 8,
+                               pango_color->blue >> 8,
+                               0xff);
 
-      cogl_material_set_color4ub (priv->solid_material,
-                                  red, green, blue, alpha);
-      cogl_material_set_color4ub (priv->glyph_material,
-                                  red, green, blue, alpha);
+      _cogl_pango_display_list_set_color_override (priv->display_list, &color);
     }
+  else
+    _cogl_pango_display_list_remove_color_override (priv->display_list);
 }
 
 static void
@@ -460,12 +518,13 @@ cogl_pango_renderer_draw_box (PangoRenderer *renderer,
 {
   CoglPangoRenderer *priv = COGL_PANGO_RENDERER (renderer);
 
-  cogl_set_source (priv->solid_material);
-  cogl_path_rectangle (x,
-		       y - height,
-		       x + width,
-		       y);
-  cogl_path_stroke ();
+  g_return_if_fail (priv->display_list != NULL);
+
+  _cogl_pango_display_list_add_rectangle (priv->display_list,
+                                          x,
+                                          y - height,
+                                          x + width,
+                                          y);
 }
 
 static void
@@ -503,6 +562,8 @@ cogl_pango_renderer_draw_rectangle (PangoRenderer   *renderer,
   CoglPangoRenderer *priv = COGL_PANGO_RENDERER (renderer);
   float x1, x2, y1, y2;
 
+  g_return_if_fail (priv->display_list != NULL);
+
   cogl_pango_renderer_set_color_for_part (renderer, part);
 
   cogl_pango_renderer_get_device_units (renderer,
@@ -512,8 +573,8 @@ cogl_pango_renderer_draw_rectangle (PangoRenderer   *renderer,
                                         x + width, y + height,
                                         &x2, &y2);
 
-  cogl_set_source (priv->solid_material);
-  cogl_rectangle (x1, y1, x2, y2);
+  _cogl_pango_display_list_add_rectangle (priv->display_list,
+                                          x1, y1, x2, y2);
 }
 
 static void
@@ -529,6 +590,8 @@ cogl_pango_renderer_draw_trapezoid (PangoRenderer   *renderer,
   CoglPangoRenderer *priv = COGL_PANGO_RENDERER (renderer);
   float points[8];
 
+  g_return_if_fail (priv->display_list != NULL);
+
   points[0] =  (x11);
   points[1] =  (y1);
   points[2] =  (x12);
@@ -540,9 +603,13 @@ cogl_pango_renderer_draw_trapezoid (PangoRenderer   *renderer,
 
   cogl_pango_renderer_set_color_for_part (renderer, part);
 
-  cogl_set_source (priv->solid_material);
-  cogl_path_polygon (points, 4);
-  cogl_path_fill ();
+  _cogl_pango_display_list_add_trapezoid (priv->display_list,
+                                          y1,
+                                          x11,
+                                          x21,
+                                          y2,
+                                          x12,
+                                          x22);
 }
 
 static void
@@ -573,8 +640,6 @@ cogl_pango_renderer_draw_glyphs (PangoRenderer    *renderer,
 	{
 	  PangoFontMetrics *metrics;
 
-          cogl_pango_renderer_glyphs_end (priv);
-
 	  if (font == NULL ||
               (metrics = pango_font_get_metrics (font, NULL)) == NULL)
             {
@@ -606,15 +671,11 @@ cogl_pango_renderer_draw_glyphs (PangoRenderer    *renderer,
                                                   gi->glyph);
 
 	  if (cache_value == NULL)
-            {
-              cogl_pango_renderer_glyphs_end (priv);
-
-              cogl_pango_renderer_draw_box (renderer,
-                                            x,
-                                            y,
-                                            PANGO_UNKNOWN_GLYPH_WIDTH,
-                                            PANGO_UNKNOWN_GLYPH_HEIGHT);
-            }
+            cogl_pango_renderer_draw_box (renderer,
+                                          x,
+                                          y,
+                                          PANGO_UNKNOWN_GLYPH_WIDTH,
+                                          PANGO_UNKNOWN_GLYPH_HEIGHT);
 	  else
 	    {
               float width, height;
@@ -631,6 +692,4 @@ cogl_pango_renderer_draw_glyphs (PangoRenderer    *renderer,
 
       xi += gi->geometry.width;
     }
-
-  cogl_pango_renderer_glyphs_end (priv);
 }
