@@ -39,8 +39,13 @@
 #include "clutter-backend-x11.h"
 #include "clutter-stage-x11.h"
 #include "clutter-x11.h"
+
 #include <X11/extensions/Xcomposite.h>
 
+#ifdef HAVE_XINPUT
+#include <X11/extensions/XInput.h>
+#endif
+
 #include "../clutter-event.h"
 #include "../clutter-main.h"
 #include "../clutter-debug.h"
@@ -53,15 +58,15 @@ G_DEFINE_TYPE (ClutterBackendX11, clutter_backend_x11, CLUTTER_TYPE_BACKEND);
 struct _ClutterX11XInputDevice
 {
   ClutterInputDevice device;
-#ifdef USE_XINPUT
+
+#ifdef HAVE_XINPUT
   XDevice           *xdevice;
   XEventClass        xevent_list[5];   /* MAX 5 event types */
   int                num_events;
 #endif
-  ClutterX11InputDeviceType type; /* FIXME: generic to ClutterInputDevice? */
 };
 
-#ifdef USE_XINPUT
+#ifdef HAVE_XINPUT
 void  _clutter_x11_register_xinput ();
 #endif
 
@@ -90,12 +95,12 @@ static ClutterBackendX11 *backend_singleton = NULL;
 
 /* various flags corresponding to pre init setup calls */
 static gboolean _no_xevent_retrieval = FALSE;
-static gboolean _enable_xinput = FALSE;
+static gboolean clutter_enable_xinput = FALSE;
 static Display  *_foreign_dpy = NULL;
 
 /* options */
 static gchar *clutter_display_name = NULL;
-static gint clutter_screen = 0;
+static gint clutter_screen = -1;
 static gboolean clutter_synchronise = FALSE;
 
 /* X error trap */
@@ -137,14 +142,14 @@ clutter_backend_x11_post_parse (ClutterBackend  *backend,
     {
       if (clutter_display_name)
 	{
-	  CLUTTER_NOTE (BACKEND, "XOpenDisplay on `%s'",
+	  CLUTTER_NOTE (BACKEND, "XOpenDisplay on '%s'",
 			clutter_display_name);
 	  backend_x11->xdpy = XOpenDisplay (clutter_display_name);
           if (backend_x11->xdpy == None)
             {
               g_set_error (error, CLUTTER_INIT_ERROR,
                            CLUTTER_INIT_ERROR_BACKEND,
-                           "Unable to open display `%s'",
+                           "Unable to open display '%s'",
                            clutter_display_name);
               return FALSE;
             }
@@ -167,7 +172,7 @@ clutter_backend_x11_post_parse (ClutterBackend  *backend,
 
       CLUTTER_NOTE (BACKEND, "Getting the X screen");
 
-      if (clutter_screen == 0)
+      if (clutter_screen == -1)
         backend_x11->xscreen = DefaultScreenOfDisplay (backend_x11->xdpy);
       else
         backend_x11->xscreen = ScreenOfDisplay (backend_x11->xdpy,
@@ -185,7 +190,7 @@ clutter_backend_x11_post_parse (ClutterBackend  *backend,
 
       clutter_backend_set_resolution (backend, dpi);
 
-#ifdef USE_XINPUT
+#ifdef HAVE_XINPUT
       _clutter_x11_register_xinput ();
 #endif
 
@@ -211,7 +216,7 @@ clutter_backend_x11_post_parse (ClutterBackend  *backend,
   g_free (clutter_display_name);
 
   CLUTTER_NOTE (BACKEND,
-                "X Display `%s'[%p] opened (screen:%d, root:%u, dpi:%f)",
+                "X Display '%s'[%p] opened (screen:%d, root:%u, dpi:%f)",
                 backend_x11->display_name,
                 backend_x11->xdpy,
                 backend_x11->xscreen_num,
@@ -248,8 +253,16 @@ static const GOptionEntry entries[] =
   { "synch", 0,
     0,
     G_OPTION_ARG_NONE, &clutter_synchronise,
-    N_("Make X calls synchronous"), NULL,
+    N_("Make X calls synchronous"), NULL
   },
+#ifdef HAVE_XINPUT
+  {
+    "enable-xinput", 0,
+    0,
+    G_OPTION_ARG_NONE, &clutter_enable_xinput,
+    N_("Enable XInput support"), NULL
+  },
+#endif /* HAVE_XINPUT */
   { NULL }
 };
 
@@ -279,13 +292,10 @@ static void
 clutter_backend_x11_dispose (GObject *gobject)
 {
   ClutterBackendX11   *backend_x11 = CLUTTER_BACKEND_X11 (gobject);
-  ClutterMainContext  *context;
   ClutterStageManager *stage_manager;
 
   CLUTTER_NOTE (BACKEND, "Disposing the of stages");
-
-  context = clutter_context_get_default ();
-  stage_manager = context->stage_manager;
+  stage_manager = clutter_stage_manager_get_default ();
 
   /* Destroy all of the stages. g_slist_foreach is used because the
      finalizer for the stages will remove the stage from the
@@ -437,12 +447,10 @@ clutter_x11_get_default_display (void)
 void
 clutter_x11_set_display (Display *xdpy)
 {
-  ClutterMainContext *ctx = clutter_context_get_default ();
-
-  if (ctx->is_initialized)
+  if (_clutter_context_is_initialized ())
     {
       g_critical ("Display connection already exists. You can only call "
-		  "clutter_x11_set_display() once before clutter_init()\n");
+		  "clutter_x11_set_display() before clutter_init()");
       return;
     }
 
@@ -462,18 +470,16 @@ clutter_x11_set_display (Display *xdpy)
  * Since: 0.8
  */
 void
-clutter_x11_enable_xinput ()
+clutter_x11_enable_xinput (void)
 {
-  ClutterMainContext *ctx = clutter_context_get_default ();
-
-  if (ctx->is_initialized)
+  if (_clutter_context_is_initialized ())
     {
-      g_warning  ("clutter_x11_enable_xinput should "
-                  "be called before clutter_init");
+      g_critical ("clutter_x11_enable_xinput() can only be called "
+                  "before clutter_init()");
       return;
     }
 
-  _enable_xinput = TRUE;
+  clutter_enable_xinput = TRUE;
 }
 
 /**
@@ -489,12 +495,10 @@ clutter_x11_enable_xinput ()
 void
 clutter_x11_disable_event_retrieval (void)
 {
-  ClutterMainContext *ctx = clutter_context_get_default ();
-
-  if (ctx->is_initialized)
+  if (_clutter_context_is_initialized ())
     {
-      g_warning  ("clutter_x11_disable_event_retrieval should "
-                  "be called before clutter_init");
+      g_warning  ("clutter_x11_disable_event_retrieval() can only be "
+                  "called before clutter_init()");
       return;
     }
 
@@ -630,19 +634,19 @@ clutter_x11_remove_filter (ClutterX11FilterFunc func,
     }
 }
 
-#ifdef USE_XINPUT
-
 void
 _clutter_x11_register_xinput ()
 {
+#ifdef HAVE_XINPUT
   XDeviceInfo *xdevices = NULL;
   XDeviceInfo *info = NULL;
 
   XDevice *xdevice = NULL;
 
   XInputClassInfo *xclass_info = NULL;
-  XExtensionVersion *ext;
 
+  gint opcode, event, error;
+  gint res;
   gint num_devices = 0;
   gint num_events = 0;
   gint i = 0, j = 0;
@@ -653,27 +657,31 @@ _clutter_x11_register_xinput ()
 
   ClutterMainContext  *context;
 
+  GSList *input_devices = NULL;
+
   if (!backend_singleton)
     {
       g_critical ("X11 backend has not been initialised");
       return;
     }
 
-  if (!_enable_xinput)
+  if (!clutter_enable_xinput)
     {
       CLUTTER_NOTE (BACKEND, "Not enabling XInput");
       return;
     }
 
-  context = clutter_context_get_default ();
+  context = _clutter_context_get_default ();
 
-  backend_singleton->have_xinput = TRUE;
+  backend_singleton->have_xinput = FALSE;
 
-  ext = XGetExtensionVersion(backend_singleton->xdpy, INAME);
-
-  if (!ext || (ext == (XExtensionVersion*) NoSuchExtension))
+  /* is the XInput extension available? */
+  res = XQueryExtension (backend_singleton->xdpy, "XInputExtension",
+                         &opcode, &event,
+                         &error);
+  if (!res)
     {
-      backend_singleton->have_xinput = FALSE;
+      CLUTTER_NOTE (BACKEND, "X Input extension not available");
       return;
     }
 
@@ -684,129 +692,146 @@ _clutter_x11_register_xinput ()
   CLUTTER_NOTE (BACKEND, "%d XINPUT devices found", num_devices);
 
   if (num_devices == 0)
-    {
-      backend_singleton->have_xinput = FALSE;
-      return;
-    }
+    return;
 
   for (i = 0; i < num_devices; i++)
-  {
-    num_events = 0;
-    info = xdevices + i;
-
-    CLUTTER_NOTE (BACKEND, "Considering %li with type %d",
-                  info->id, info->use);
-
-    /* Only want 'raw' devices themselves not virtual ones */
-    if (info->use == IsXExtensionPointer ||
-        /*info->use == IsXExtensionKeyboard || XInput is broken */
-        info->use == IsXExtensionDevice)
     {
-      clutter_x11_trap_x_errors ();
-      xdevice = XOpenDevice (x11b->xdpy, info->id);
-      if (clutter_x11_untrap_x_errors () || xdevice == NULL)
-        continue;
+      info = xdevices + i;
+      num_events = 0;
 
-      /* Create the appropriate Clutter device */
-      device = g_new0 (ClutterX11XInputDevice, 1);
-      context->input_devices = g_slist_append (context->input_devices, device);
+      CLUTTER_NOTE (BACKEND, "Considering %li with type %d",
+                    info->id,
+                    info->use);
 
-      device->device.id = info->id;
-
-      /* FIXME: some kind of general device_init() call should do below */
-      device->device.click_count = 0;
-      device->device.previous_time = 0;
-      device->device.previous_x = -1;
-      device->device.previous_y = -1;
-      device->device.previous_button_number = -1;
-
-      device->xdevice = xdevice;
-      device->num_events = 0;
-
-      switch (info->use)
-      {
-        case IsXExtensionPointer:
-          device->type = CLUTTER_X11_XINPUT_POINTER_DEVICE;
-          have_an_xpointer = TRUE;
-          break;
-        /* XInput is broken:
-        case IsXExtensionKeyboard:
-          device->type = CLUTTER_X11_XINPUT_KEYBOARD_DEVICE;
-          break;*/
-        case IsXExtensionDevice:
-          device->type = CLUTTER_X11_XINPUT_EXTENSION_DEVICE;
-          break;
-      }
-
-      CLUTTER_NOTE (BACKEND, "Registering XINPUT device with XID: %li",
-                    xdevice->device_id);
-
-      /* We must go through all the classes supported by this device and
-       * register the appropriate events we want. Each class only appears
-       * once. We need to store the types with the stage since they are
-       * created dynamically by the server. They are not device specific.
-       */
-      for (j = 0; j < xdevice->num_classes; j++)
-      {
-        xclass_info = xdevice->classes + j;
-
-        switch (xclass_info->input_class)
+      /* Only want 'raw' devices themselves not virtual ones */
+      if (info->use == IsXExtensionPointer ||
+        /*info->use == IsXExtensionKeyboard || XInput 1.x is broken */
+          info->use == IsXExtensionDevice)
         {
-#if 0
-      /* We do not do XInput keyboard events yet, since it is broken */
-          case KeyClass:
-            DeviceKeyPress (xdevice,
-                x11b->event_types [CLUTTER_X11_XINPUT_KEY_PRESS_EVENT],
-                device->xevent_list [num_events]);
-            num_events++;
+          clutter_x11_trap_x_errors ();
+          xdevice = XOpenDevice (backend_singleton->xdpy, info->id);
+          if (clutter_x11_untrap_x_errors () || xdevice == NULL)
+            continue;
 
-            DeviceKeyRelease (xdevice,
-                x11b->event_types [CLUTTER_X11_XINPUT_KEY_RELEASE_EVENT],
-                device->xevent_list [num_events]);
-            num_events++;
-            break;
+          /* Create the appropriate Clutter device */
+          device = g_slice_new0 (ClutterX11XInputDevice);
+
+          device->device.id = info->id;
+
+          switch (info->use)
+            {
+            case IsXExtensionPointer:
+              device->device.device_type = CLUTTER_POINTER_DEVICE;
+              have_an_xpointer = TRUE;
+              break;
+
+#if 0
+            /* XInput 1.x is broken for keyboards: */
+            case IsXExtensionKeyboard:
+              device->device.type = CLUTTER_KEYBOARD_DEVICE;
+              break;
 #endif
 
-          case ButtonClass:
-            DeviceButtonPress (xdevice,
-                x11b->event_types [CLUTTER_X11_XINPUT_BUTTON_PRESS_EVENT],
-                device->xevent_list [num_events]);
-            num_events++;
+            case IsXExtensionDevice:
+              device->device.device_type = CLUTTER_EXTENSION_DEVICE;
+              break;
+            }
 
-            DeviceButtonRelease (xdevice,
-                x11b->event_types [CLUTTER_X11_XINPUT_BUTTON_RELEASE_EVENT],
-                device->xevent_list [num_events]);
-            num_events++;
-            break;
+          /* FIXME: some kind of general device_init() call should do below */
+          device->device.click_count = 0;
+          device->device.previous_time = 0;
+          device->device.previous_x = -1;
+          device->device.previous_y = -1;
+          device->device.previous_button_number = -1;
 
-          case ValuatorClass:
-            DeviceMotionNotify (xdevice,
-                x11b->event_types [CLUTTER_X11_XINPUT_MOTION_NOTIFY_EVENT],
-                                   device->xevent_list [num_events]);
-            num_events++;
-            break;
+          device->num_events = 0;
+          device->xdevice = xdevice;
+
+          CLUTTER_NOTE (BACKEND, "Registering XINPUT device with XID: %li",
+                        xdevice->device_id);
+
+          /* We must go through all the classes supported by this device and
+           * register the appropriate events we want. Each class only appears
+           * once. We need to store the types with the stage since they are
+           * created dynamically by the server. They are not device specific.
+           */
+          for (j = 0; j < xdevice->num_classes; j++)
+            {
+              xclass_info = xdevice->classes + j;
+
+              switch (xclass_info->input_class)
+                {
+#if 0
+                /* XInput 1.x is broken for keyboards: */
+                case KeyClass:
+                  DeviceKeyPress (xdevice,
+                                  x11b->event_types[CLUTTER_X11_XINPUT_KEY_PRESS_EVENT],
+                                  device->xevent_list [num_events]);
+                  num_events++;
+
+                  DeviceKeyRelease (xdevice,
+                                    x11b->event_types[CLUTTER_X11_XINPUT_KEY_RELEASE_EVENT],
+                                    device->xevent_list [num_events]);
+                  num_events++;
+                  break;
+#endif
+
+                case ButtonClass:
+                  DeviceButtonPress (xdevice,
+                                     x11b->event_types[CLUTTER_X11_XINPUT_BUTTON_PRESS_EVENT],
+                                     device->xevent_list [num_events]);
+                  num_events++;
+
+                  DeviceButtonRelease (xdevice,
+                                       x11b->event_types[CLUTTER_X11_XINPUT_BUTTON_RELEASE_EVENT],
+                                       device->xevent_list [num_events]);
+                  num_events++;
+                  break;
+
+                case ValuatorClass:
+                  DeviceMotionNotify (xdevice,
+                                      x11b->event_types[CLUTTER_X11_XINPUT_MOTION_NOTIFY_EVENT],
+                                      device->xevent_list [num_events]);
+                  num_events++;
+                  break;
+                }
+            }
+
+          if (info->use == IsXExtensionPointer && num_events > 0)
+            have_an_xpointer = TRUE;
+
+          device->num_events  = num_events;
+
+          input_devices = g_slist_prepend (input_devices, device);
         }
-      }
-
-      if (info->use == IsXExtensionPointer && num_events > 0)
-        have_an_xpointer = TRUE;
-
-      device->num_events  = num_events;
     }
-  }
 
   XFree (xdevices);
 
   if (!have_an_xpointer)
     {
+      GSList *l;
+
       /* Something is likely wrong with Xinput setup so we basically
        * abort here and fall back to lofi regular xinput.
       */
       g_warning ("No usuable XInput pointing devices found");
-      backend_singleton->have_xinput = FALSE;
-      g_slist_free (context->input_devices);
+
+      for (l = input_devices; l != NULL; l = l->next)
+        g_slice_free (ClutterX11XInputDevice, l->data);
+
+      g_slist_free (input_devices);
       context->input_devices = NULL;
+
+      return;
     }
+
+  /* store the list of input devices */
+  context->input_devices = g_slist_reverse (input_devices);
+
+  /* why yes, we are awesome */
+  backend_singleton->have_xinput = TRUE;
+#endif /* HAVE_XINPUT */
 }
 
 void
@@ -818,12 +843,13 @@ _clutter_x11_unregister_xinput ()
 void
 _clutter_x11_select_events (Window xwin)
 {
+#ifdef HAVE_XINPUT
   GSList *list_it;
   ClutterX11XInputDevice *device = NULL;
 
   ClutterMainContext  *context;
 
-  context = clutter_context_get_default ();
+  context = _clutter_context_get_default ();
 
   if (!backend_singleton)
     {
@@ -842,16 +868,17 @@ _clutter_x11_select_events (Window xwin)
                            device->xevent_list,
                            device->num_events);
   }
+#endif /* HAVE_XINPUT */
 }
 
-ClutterX11XInputDevice *
+ClutterInputDevice *
 _clutter_x11_get_device_for_xid (XID id)
 {
-  GSList *list_it;
-  ClutterX11XInputDevice *device = NULL;
-  ClutterMainContext  *context;
+#ifdef HAVE_XINPUT
+  ClutterMainContext *context;
+  GSList *l;
 
-  context = clutter_context_get_default ();
+  context = _clutter_context_get_default ();
 
   if (!backend_singleton)
     {
@@ -859,55 +886,37 @@ _clutter_x11_get_device_for_xid (XID id)
       return NULL;
     }
 
-  for (list_it = context->input_devices;
-       list_it != NULL;
-       list_it = list_it->next)
-  {
-    device = (ClutterX11XInputDevice *)list_it->data;
+  for (l = context->input_devices; l != NULL; l = l->next)
+    {
+      ClutterX11XInputDevice *device = l->data;
 
-    if (device->xdevice->device_id == id)
-      return device;
-  }
+      if (device->xdevice->device_id == id)
+        return (ClutterInputDevice *) device;
+    }
+#endif /* HAVE_XINPUT */
 
   return NULL;
 }
-#endif
 
 /* FIXME: This nasty little func needs moving elsewhere.. */
-GSList*
+G_CONST_RETURN GSList *
 clutter_x11_get_input_devices (void)
 {
+#ifdef HAVE_XINPUT
   ClutterMainContext  *context;
 
-#ifdef USE_XINPUT
   if (!backend_singleton)
     {
       g_critical ("X11 backend has not been initialised");
       return NULL;
     }
 
-  context = clutter_context_get_default ();
+  context = _clutter_context_get_default ();
 
   return context->input_devices;
-#else
+#else /* !HAVE_XINPUT */
   return NULL;
-#endif
-}
-
-/**
- * clutter_x11_get_input_device_type:
- * @device: a #ClutterX11XInputDevice
- *
- * Retrieves the type of @device.
- *
- * Return value: the type of the device
- *
- * Since: 0.8
- */
-ClutterX11InputDeviceType
-clutter_x11_get_input_device_type (ClutterX11XInputDevice *device)
-{
-  return device->type;
+#endif /* HAVE_XINPUT */
 }
 
 /**
@@ -923,7 +932,7 @@ clutter_x11_get_input_device_type (ClutterX11XInputDevice *device)
 gboolean
 clutter_x11_has_xinput (void)
 {
-#ifdef USE_XINPUT
+#ifdef HAVE_XINPUT
   if (!backend_singleton)
     {
       g_critical ("X11 backend has not been initialised");
@@ -939,9 +948,9 @@ clutter_x11_has_xinput (void)
 gboolean
 clutter_x11_has_composite_extension (void)
 {
-  static gboolean                 have_composite = FALSE, done_check = FALSE;
-  int                             error = 0, event = 0;
-  Display                        *dpy;
+  static gboolean have_composite = FALSE, done_check = FALSE;
+  int error = 0, event = 0;
+  Display *dpy;
 
   if (done_check)
     return have_composite;
@@ -969,3 +978,17 @@ clutter_x11_has_composite_extension (void)
   return have_composite;
 }
 
+XVisualInfo *
+clutter_backend_x11_get_visual_info (ClutterBackendX11 *backend_x11,
+                                     gboolean           for_offscreen)
+{
+  ClutterBackendX11Class *klass;
+
+  g_return_val_if_fail (CLUTTER_IS_BACKEND_X11 (backend_x11), NULL);
+
+  klass = CLUTTER_BACKEND_X11_GET_CLASS (backend_x11);
+  if (klass->get_visual_info)
+    return klass->get_visual_info (backend_x11, for_offscreen);
+
+  return NULL;
+}
