@@ -531,8 +531,6 @@ clutter_x11_texture_pixmap_realize (ClutterActor *actor)
   CLUTTER_ACTOR_CLASS (clutter_x11_texture_pixmap_parent_class)->
       realize (actor);
 
-  CLUTTER_ACTOR_SET_FLAGS (actor, CLUTTER_ACTOR_REALIZED);
-
   clutter_x11_texture_pixmap_update_area_real (texture,
 					       0, 0,
 					       priv->pixmap_width,
@@ -873,7 +871,8 @@ clutter_x11_texture_pixmap_update_area_real (ClutterX11TexturePixmap *texture,
 					     width, height,
 					     bytes_per_line,
 					     4,
-					     CLUTTER_TEXTURE_RGB_FLAG_BGR,
+					     CLUTTER_TEXTURE_RGB_FLAG_BGR |
+					     CLUTTER_TEXTURE_RGB_FLAG_PREMULT,
 					     &error);
   else
     clutter_texture_set_from_rgb_data  (CLUTTER_TEXTURE (texture),
@@ -882,7 +881,8 @@ clutter_x11_texture_pixmap_update_area_real (ClutterX11TexturePixmap *texture,
 					width, height,
 					bytes_per_line,
 					4,
-					CLUTTER_TEXTURE_RGB_FLAG_BGR,
+					CLUTTER_TEXTURE_RGB_FLAG_BGR |
+					CLUTTER_TEXTURE_RGB_FLAG_PREMULT,
 					&error);
 
 
@@ -921,9 +921,6 @@ clutter_x11_texture_pixmap_new (void)
 /**
  * clutter_x11_texture_pixmap_new_with_pixmap:
  * @pixmap: the X Pixmap to which this texture should be bound
- * @width: the width of the X pixmap
- * @height: the height of the X pixmap
- * @depth: the depth of the X pixmap
  *
  * Return value: A new #ClutterX11TexturePixmap bound to the given X Pixmap
  *
@@ -944,9 +941,6 @@ clutter_x11_texture_pixmap_new_with_pixmap (Pixmap pixmap)
 /**
  * clutter_x11_texture_pixmap_new_with_window:
  * @window: the X window to which this texture should be bound
- * @width: the width of the X pixmap
- * @height: the height of the X pixmap
- * @depth: the depth of the X pixmap
  *
  * Return value: A new #ClutterX11TexturePixmap bound to the given X window.
  *
@@ -1202,32 +1196,40 @@ clutter_x11_texture_pixmap_sync_window (ClutterX11TexturePixmap *texture)
     {
       XWindowAttributes attr;
       Display *dpy = clutter_x11_get_default_display ();
-      gboolean mapped, notify_x, notify_y, notify_override_redirect;
+      gboolean mapped = FALSE;
+      gboolean notify_x = FALSE;
+      gboolean notify_y = FALSE;
+      gboolean notify_override_redirect = FALSE;
+      Status status;
 
-      /*
-       * Make sure the window is mapped when getting the pixmap -- it's what
-       * compiz does.
+      /* NB: It's only valid to name a pixmap if the window is viewable.
+       *
+       * We don't explicitly check this though since there would be a race
+       * between checking and naming unless we use a server grab which is
+       * undesireable.
+       *
+       * Instead we gracefully handle any error with naming the pixmap.
        */
-
       clutter_x11_trap_x_errors ();
-      XGrabServer (dpy);
 
-      XGetWindowAttributes (dpy, priv->window, &attr);
-      mapped = attr.map_state == IsViewable;
-      if (mapped)
-        pixmap = XCompositeNameWindowPixmap (dpy, priv->window);
-      else
-        pixmap = None;
+      pixmap = XCompositeNameWindowPixmap (dpy, priv->window);
 
-      XUngrabServer (dpy);
-      clutter_x11_untrap_x_errors ();
+      status = XGetWindowAttributes (dpy, priv->window, &attr);
+      if (status != 0)
+	{
+	  notify_x = attr.x != priv->window_x;
+	  notify_y = attr.y != priv->window_y;
+	  notify_override_redirect = attr.override_redirect != priv->override_redirect;
+	  priv->window_x = attr.x;
+	  priv->window_y = attr.y;
+	  priv->override_redirect = attr.override_redirect;
+	}
 
-      notify_x = attr.x != priv->window_x;
-      notify_y = attr.y != priv->window_y;
-      notify_override_redirect = attr.override_redirect != priv->override_redirect;
-      priv->window_x = attr.x;
-      priv->window_y = attr.y;
-      priv->override_redirect = attr.override_redirect;
+      /* We rely on XGetWindowAttributes() implicitly syncing with the server
+       * so if there was an error naming the pixmap that will have been
+       * caught by now. */
+      if (clutter_x11_untrap_x_errors ())
+	pixmap = None;
 
       g_object_ref (texture); /* guard against unparent */
       if (pixmap)
@@ -1304,9 +1306,6 @@ clutter_x11_texture_pixmap_update_area (ClutterX11TexturePixmap *texture,
 {
   g_return_if_fail (CLUTTER_X11_IS_TEXTURE_PIXMAP (texture));
 
-  if (!CLUTTER_ACTOR_IS_VISIBLE (CLUTTER_ACTOR(texture)))
-    return; /* No need to update when not visible */
-
   g_signal_emit (texture, signals[UPDATE_AREA], 0, x, y, width, height);
 }
 
