@@ -60,11 +60,10 @@ int
 main (int argc, char **argv)
 {
   TestConformSharedState *shared_state = g_new0 (TestConformSharedState, 1);
-  const gchar *display;
 
 #ifdef HAVE_CLUTTER_GLX
   /* on X11 we need a display connection to run the test suite */
-  display = g_getenv ("DISPLAY");
+  const gchar *display = g_getenv ("DISPLAY");
   if (!display || *display == '\0')
     {
       g_print ("No DISPLAY found. Unable to run the conformance "
@@ -73,24 +72,28 @@ main (int argc, char **argv)
     }
 #endif
 
+  /* Turning of sync-to-vblank removes a dependency on the specifics of the
+   * test environment. It also means that the timeline-only tests are
+   * throttled to a reasonable frame rate rather than running in tight
+   * infinite loop.
+   */
+  g_setenv ("CLUTTER_VBLANK", "none", FALSE);
+
   g_test_init (&argc, &argv, NULL);
 
   g_test_bug_base ("http://bugzilla.openedhand.com/show_bug.cgi?id=%s");
 
-  g_assert (clutter_init (shared_state->argc_addr, shared_state->argv_addr)
-	    == CLUTTER_INIT_SUCCESS);
-
   /* Initialise the state you need to share with everything.
    */
   shared_state->argc_addr = &argc;
   shared_state->argv_addr = &argv;
 
+  g_assert (clutter_init (shared_state->argc_addr, shared_state->argv_addr)
+	    == CLUTTER_INIT_SUCCESS);
+
   TEST_CONFORM_SIMPLE ("/timeline", test_timeline);
-  TEST_CONFORM_SKIP (!g_test_slow (), "/timeline", test_timeline_dup_frames);
-  TEST_CONFORM_SKIP (!g_test_slow (), "/timeline", test_timeline_dup_frames);
   TEST_CONFORM_SKIP (!g_test_slow (), "/timeline", test_timeline_interpolate);
   TEST_CONFORM_SKIP (!g_test_slow (), "/timeline", test_timeline_rewind);
-  TEST_CONFORM_SKIP (!g_test_slow (), "/timeline", test_timeline_smoothness);
 
   TEST_CONFORM_SIMPLE ("/picking", test_pick);
 
@@ -116,9 +119,13 @@ main (int argc, char **argv)
   TEST_CONFORM_SIMPLE ("/fixed", test_fixed_constants);
 
   TEST_CONFORM_SIMPLE ("/invariants", test_initial_state);
+  TEST_CONFORM_SIMPLE ("/invariants", test_shown_not_parented);
   TEST_CONFORM_SIMPLE ("/invariants", test_realized);
+  TEST_CONFORM_SIMPLE ("/invariants", test_realize_not_recursive);
+  TEST_CONFORM_SIMPLE ("/invariants", test_map_recursive);
   TEST_CONFORM_SIMPLE ("/invariants", test_mapped);
   TEST_CONFORM_SIMPLE ("/invariants", test_show_on_set_parent);
+  TEST_CONFORM_SIMPLE ("/invariants", test_clone_no_map);
 
   TEST_CONFORM_SIMPLE ("/vertex-buffer", test_vertex_buffer_contiguous);
   TEST_CONFORM_SIMPLE ("/vertex-buffer", test_vertex_buffer_interleved);
@@ -130,6 +137,7 @@ main (int argc, char **argv)
 
   TEST_CONFORM_SIMPLE ("/texture", test_backface_culling);
   TEST_CONFORM_SIMPLE ("/texture", test_npot_texture);
+  TEST_CONFORM_SIMPLE ("/texture", test_premult);
 
   TEST_CONFORM_SIMPLE ("/path", test_path);
 
@@ -139,6 +147,16 @@ main (int argc, char **argv)
 
   TEST_CONFORM_SIMPLE ("/model", test_list_model_populate);
   TEST_CONFORM_SIMPLE ("/model", test_list_model_iterate);
+  TEST_CONFORM_SIMPLE ("/model", test_list_model_filter);
+
+  TEST_CONFORM_SIMPLE ("/material", test_materials);
+  TEST_CONFORM_SIMPLE ("/material", test_blend_strings);
+
+  TEST_CONFORM_SIMPLE ("/color", test_color_from_string);
+  TEST_CONFORM_SIMPLE ("/color", test_color_to_string);
+
+  TEST_CONFORM_SIMPLE ("/units", test_units_constructors);
+  TEST_CONFORM_SIMPLE ("/units", test_units_string);
 
   return g_test_run ();
 }
